--
-------------------------------------------------------------------------------------------
-- Copyright  2010-2013, Xilinx, Inc.
-- This file contains confidential and proprietary information of Xilinx, Inc. and is
-- protected under U.S. and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by Xilinx, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND XILINX HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) Xilinx shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or Xilinx had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- Xilinx products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of Xilinx products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
--
-------------------------------------------------------------------------------------------
--
--
-- Definition of a program memory for KCPSM6 including generic parameters for the 
-- convenient selection of device family, program memory size and the ability to include 
-- the JTAG Loader hardware for rapid software development.
--
-- This file is primarily for use during code development and it is recommended that the 
-- appropriate simplified program memory definition be used in a final production design. 
--
--    Generic                  Values             Comments
--    Parameter                Supported
--  
--    C_FAMILY                 "S6"               Spartan-6 device
--                             "V6"               Virtex-6 device
--                             "7S"               7-Series device 
--                                                  (Artix-7, Kintex-7, Virtex-7 or Zynq)
--
--    C_RAM_SIZE_KWORDS        1, 2 or 4          Size of program memory in K-instructions
--
--    C_JTAG_LOADER_ENABLE     0 or 1             Set to '1' to include JTAG Loader
--
-- Notes
--
-- If your design contains MULTIPLE KCPSM6 instances then only one should have the 
-- JTAG Loader enabled at a time (i.e. make sure that C_JTAG_LOADER_ENABLE is only set to 
-- '1' on one instance of the program memory). Advanced users may be interested to know 
-- that it is possible to connect JTAG Loader to multiple memories and then to use the 
-- JTAG Loader utility to specify which memory contents are to be modified. However, 
-- this scheme does require some effort to set up and the additional connectivity of the 
-- multiple BRAMs can impact the placement, routing and performance of the complete 
-- design. Please contact the author at Xilinx for more detailed information. 
--
-- Regardless of the size of program memory specified by C_RAM_SIZE_KWORDS, the complete 
-- 12-bit address bus is connected to KCPSM6. This enables the generic to be modified 
-- without requiring changes to the fundamental hardware definition. However, when the 
-- program memory is 1K then only the lower 10-bits of the address are actually used and 
-- the valid address range is 000 to 3FF hex. Likewise, for a 2K program only the lower 
-- 11-bits of the address are actually used and the valid address range is 000 to 7FF hex.
--
-- Programs are stored in Block Memory (BRAM) and the number of BRAM used depends on the 
-- size of the program and the device family. 
--
-- In a Spartan-6 device a BRAM is capable of holding 1K instructions. Hence a 2K program 
-- will require 2 BRAMs to be used and a 4K program will require 4 BRAMs to be used. It 
-- should be noted that a 4K program is not such a natural fit in a Spartan-6 device and 
-- the implementation also requires a small amount of logic resulting in slightly lower 
-- performance. A Spartan-6 BRAM can also be split into two 9k-bit memories suggesting 
-- that a program containing up to 512 instructions could be implemented. However, there 
-- is a silicon errata which makes this unsuitable and therefore it is not supported by 
-- this file.
--
-- In a Virtex-6 or any 7-Series device a BRAM is capable of holding 2K instructions so 
-- obviously a 2K program requires only a single BRAM. Each BRAM can also be divided into 
-- 2 smaller memories supporting programs of 1K in half of a 36k-bit BRAM (generally 
-- reported as being an 18k-bit BRAM). For a program of 4K instructions, 2 BRAMs are used.
--
--
-- Program defined by '{psmname}.psm'.
--
-- Generated by KCPSM6 Assembler: 2016-10-10T13:58:29. 
--
-- Assembler used ROM_form template: ROM_form_JTAGLoader_14March13.vhd
--
-- Standard IEEE libraries
--
--
package jtag_loader_pkg is
 function addr_width_calc (size_in_k: integer) return integer;
end jtag_loader_pkg;
--
package body jtag_loader_pkg is
  function addr_width_calc (size_in_k: integer) return integer is
   begin
    if (size_in_k = 1) then return 10;
      elsif (size_in_k = 2) then return 11;
      elsif (size_in_k = 4) then return 12;
      else report "Invalid BlockRAM size. Please set to 1, 2 or 4 K words." severity FAILURE;
    end if;
    return 0;
  end function addr_width_calc;
end package body;
--
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use work.jtag_loader_pkg.ALL;
--
-- The Unisim Library is used to define Xilinx primitives. It is also used during
-- simulation. The source can be viewed at %XILINX%\vhdl\src\unisims\unisim_VCOMP.vhd
--  
library unisim;
use unisim.vcomponents.all;
--
--
entity m25p16_spi_uart_bridge is
  generic(             C_FAMILY : string := "S6"; 
              C_RAM_SIZE_KWORDS : integer := 1;
           C_JTAG_LOADER_ENABLE : integer := 0);
  Port (      address : in std_logic_vector(11 downto 0);
          instruction : out std_logic_vector(17 downto 0);
               enable : in std_logic;
                  rdl : out std_logic;                    
                  clk : in std_logic);
  end m25p16_spi_uart_bridge;
--
architecture low_level_definition of m25p16_spi_uart_bridge is
--
signal       address_a : std_logic_vector(15 downto 0);
signal        pipe_a11 : std_logic;
signal       data_in_a : std_logic_vector(35 downto 0);
signal      data_out_a : std_logic_vector(35 downto 0);
signal    data_out_a_l : std_logic_vector(35 downto 0);
signal    data_out_a_h : std_logic_vector(35 downto 0);
signal   data_out_a_ll : std_logic_vector(35 downto 0);
signal   data_out_a_lh : std_logic_vector(35 downto 0);
signal   data_out_a_hl : std_logic_vector(35 downto 0);
signal   data_out_a_hh : std_logic_vector(35 downto 0);
signal       address_b : std_logic_vector(15 downto 0);
signal       data_in_b : std_logic_vector(35 downto 0);
signal     data_in_b_l : std_logic_vector(35 downto 0);
signal    data_in_b_ll : std_logic_vector(35 downto 0);
signal    data_in_b_hl : std_logic_vector(35 downto 0);
signal      data_out_b : std_logic_vector(35 downto 0);
signal    data_out_b_l : std_logic_vector(35 downto 0);
signal   data_out_b_ll : std_logic_vector(35 downto 0);
signal   data_out_b_hl : std_logic_vector(35 downto 0);
signal     data_in_b_h : std_logic_vector(35 downto 0);
signal    data_in_b_lh : std_logic_vector(35 downto 0);
signal    data_in_b_hh : std_logic_vector(35 downto 0);
signal    data_out_b_h : std_logic_vector(35 downto 0);
signal   data_out_b_lh : std_logic_vector(35 downto 0);
signal   data_out_b_hh : std_logic_vector(35 downto 0);
signal        enable_b : std_logic;
signal           clk_b : std_logic;
signal            we_b : std_logic_vector(7 downto 0);
signal          we_b_l : std_logic_vector(3 downto 0);
signal          we_b_h : std_logic_vector(3 downto 0);
-- 
signal       jtag_addr : std_logic_vector(11 downto 0);
signal         jtag_we : std_logic;
signal       jtag_we_l : std_logic;
signal       jtag_we_h : std_logic;
signal        jtag_clk : std_logic;
signal        jtag_din : std_logic_vector(17 downto 0);
signal       jtag_dout : std_logic_vector(17 downto 0);
signal     jtag_dout_1 : std_logic_vector(17 downto 0);
signal         jtag_en : std_logic_vector(0 downto 0);
-- 
signal picoblaze_reset : std_logic_vector(0 downto 0);
signal         rdl_bus : std_logic_vector(0 downto 0);
--
constant BRAM_ADDRESS_WIDTH  : integer := addr_width_calc(C_RAM_SIZE_KWORDS);
--
--
component jtag_loader_6
generic(                C_JTAG_LOADER_ENABLE : integer := 1;
                                    C_FAMILY : string  := "V6";
                             C_NUM_PICOBLAZE : integer := 1;
                       C_BRAM_MAX_ADDR_WIDTH : integer := 10;
          C_PICOBLAZE_INSTRUCTION_DATA_WIDTH : integer := 18;
                                C_JTAG_CHAIN : integer := 2;
                              C_ADDR_WIDTH_0 : integer := 10;
                              C_ADDR_WIDTH_1 : integer := 10;
                              C_ADDR_WIDTH_2 : integer := 10;
                              C_ADDR_WIDTH_3 : integer := 10;
                              C_ADDR_WIDTH_4 : integer := 10;
                              C_ADDR_WIDTH_5 : integer := 10;
                              C_ADDR_WIDTH_6 : integer := 10;
                              C_ADDR_WIDTH_7 : integer := 10);
port(              picoblaze_reset : out std_logic_vector(C_NUM_PICOBLAZE-1 downto 0);
                           jtag_en : out std_logic_vector(C_NUM_PICOBLAZE-1 downto 0);
                          jtag_din : out STD_LOGIC_VECTOR(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
                         jtag_addr : out STD_LOGIC_VECTOR(C_BRAM_MAX_ADDR_WIDTH-1 downto 0);
                          jtag_clk : out std_logic;
                           jtag_we : out std_logic;
                       jtag_dout_0 : in STD_LOGIC_VECTOR(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
                       jtag_dout_1 : in STD_LOGIC_VECTOR(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
                       jtag_dout_2 : in STD_LOGIC_VECTOR(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
                       jtag_dout_3 : in STD_LOGIC_VECTOR(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
                       jtag_dout_4 : in STD_LOGIC_VECTOR(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
                       jtag_dout_5 : in STD_LOGIC_VECTOR(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
                       jtag_dout_6 : in STD_LOGIC_VECTOR(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
                       jtag_dout_7 : in STD_LOGIC_VECTOR(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0));
end component;
--
begin
  --
  --  
  ram_1k_generate : if (C_RAM_SIZE_KWORDS = 1) generate
 
    s6: if (C_FAMILY = "S6") generate 
      --
      address_a(13 downto 0) <= address(9 downto 0) & "0000";
      instruction <= data_out_a(33 downto 32) & data_out_a(15 downto 0);
      data_in_a <= "0000000000000000000000000000000000" & address(11 downto 10);
      jtag_dout <= data_out_b(33 downto 32) & data_out_b(15 downto 0);
      --
      no_loader : if (C_JTAG_LOADER_ENABLE = 0) generate
        data_in_b <= "00" & data_out_b(33 downto 32) & "0000000000000000" & data_out_b(15 downto 0);
        address_b(13 downto 0) <= "00000000000000";
        we_b(3 downto 0) <= "0000";
        enable_b <= '0';
        rdl <= '0';
        clk_b <= '0';
      end generate no_loader;
      --
      loader : if (C_JTAG_LOADER_ENABLE = 1) generate
        data_in_b <= "00" & jtag_din(17 downto 16) & "0000000000000000" & jtag_din(15 downto 0);
        address_b(13 downto 0) <= jtag_addr(9 downto 0) & "0000";
        we_b(3 downto 0) <= jtag_we & jtag_we & jtag_we & jtag_we;
        enable_b <= jtag_en(0);
        rdl <= rdl_bus(0);
        clk_b <= jtag_clk;
      end generate loader;
      --
      kcpsm6_rom: RAMB16BWER
      generic map ( DATA_WIDTH_A => 18,
                    DOA_REG => 0,
                    EN_RSTRAM_A => FALSE,
                    INIT_A => X"000000000",
                    RST_PRIORITY_A => "CE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    DATA_WIDTH_B => 18,
                    DOB_REG => 0,
                    EN_RSTRAM_B => FALSE,
                    INIT_B => X"000000000",
                    RST_PRIORITY_B => "CE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    RSTTYPE => "SYNC",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    SIM_DEVICE => "SPARTAN6",
                    INIT_00 => X"D92000490820004C0920004C004C129FB001B03171FF003FB674003BB174003F",
                    INIT_01 => X"61107001039500A303C803EA601BD040900017001800190720132014F215F820",
                    INIT_02 => X"9511900095E9900015B9500035DFD000D57B9000D561202300E32013D1FF7000",
                    INIT_03 => X"129820431040110D120320431010112712005000150A500095F690001507E035",
                    INIT_04 => X"D2019205D208B6645000B674004C50006043B200B10090010000204310801196",
                    INIT_05 => X"38FB206B005338FB50000270004D0280004D0290004D1200B65450009204604E",
                    INIT_06 => X"1200B6545000B674004D00535804191F0069191E50000620006B0720004D0053",
                    INIT_07 => X"12FF004D1201004D1201004D12A8004D12C0004D1280004D1204004D1200004D",
                    INIT_08 => X"1202004D1201004D12DC004D1208004D1200004D1200004D12FF004D12FF004D",
                    INIT_09 => X"1601004DA2602110B674004D1202004D1201004D12A8004D12C0004D1203004D",
                    INIT_0A => X"004D1200004D1200B654606ED280B2005000D204005C180019395000609DC670",
                    INIT_0B => X"190CB654012DB615B71419040069B71318081900B674009D17131600004D1204",
                    INIT_0C => X"B726191D00691704191B012D16E81703191918000110B674009D171C16160053",
                    INIT_0D => X"1901003FB674003BB2742069003717101901500061101000D1FFB11C0069D701",
                    INIT_0E => X"D21820DFD21C182C2142D2172132D20020F9D214005C18281903206900371708",
                    INIT_0F => X"2308D2002350D217005C180819032137D21320D7D21D20D7D21B20D7D21A20D7",
                    INIT_10 => X"50002301D2152318D21320D7D21D20D7D21B20D7D21A20D7D21820DBD21C180C",
                    INIT_11 => X"006618C8006618A8006618880066186800661848170000661828006618081708",
                    INIT_12 => X"004D00535804165017001904710000691701190000D7182800D71808006618E8",
                    INIT_13 => X"7E007D007C007B007A0000691702182C1901203700691701182C1901206B0260",
                    INIT_14 => X"2183F700D60073006150FFFFDC0470006FF06CC07001005F1828192650007F00",
                    INIT_15 => X"1300004D1238004D0260004D0270B654005F182819280B700A601000F700D600",
                    INIT_16 => X"E5B0C4A02171D3FF2171D20A216CFFFFDC04350014011301E230004D14001500",
                    INIT_17 => X"4191DFFF7001006917401901012D182C192827900680B6740950084063306162",
                    INIT_18 => X"1000FB00DA0070006FF06CC06BB06AA06990688067706330700150007000DFFF",
                    INIT_19 => X"DD801FFE61A5F40AF50DD302B401B500500013001F014198DF0001DADF006BA0",
                    INIT_1A => X"61BEDD8061BF02D31AEC1B0269906880677061B060A050001A341B021C005000",
                    INIT_1B => X"70001D0021A1E1B41401170161BFC250A54021BED20003BF1907180017401415",
                    INIT_1C => X"10049000002B0025A5A01B0D1A07170050001E0070016990688067706B106A00",
                    INIT_1D => X"1C000D2003BF170018001907500061CBCAB01A01475061D09001490048004706",
                    INIT_1E => X"D262A1A101C850001A3E1B0217001800190661EDD220B205620502D31ADE1B02",
                    INIT_1F => X"D27250001A3E1B021C0561FFD27350001A3A1B021C0361F9D26521EA1C0261F3",
                    INIT_20 => X"700150001A0F1B021C04A1A101C861A102D31AE41B0250001A301B021C0361EA",
                    INIT_21 => X"6240A4E01E005000223E7000FE1FDDFF3E001D0170010037621DD30070006330",
                    INIT_22 => X"50001E001D0003FC221E221EDE0F222CCE301E01700039003800170103D97001",
                    INIT_23 => X"6CC07001700003FC03EA7001223E7000170018FE190670014BA01A311B02023E",
                    INIT_24 => X"1440151F1C00170018FF19066D706B806A907001005F18281924225EDC007000",
                    INIT_25 => X"18281924226BDC03226BDC041D006CC067D068B069A002C4182C02B519347000",
                    INIT_26 => X"D9010290700120DF182C02C4182C02B570001440151F70011934226DDC05005F",
                    INIT_27 => X"588018007001038770006380627FD902227E00D700DF627ED100180C01806278",
                    INIT_28 => X"700103B17000F80003B61300F702F800387F2290D700F80003B614091300F800",
                    INIT_29 => X"02A5A28079341800F20D9206F20C9207F20B005C1903F20A005C18081900F109",
                    INIT_2A => X"004D0280004D0290004D0260004D0270B6547000682022697000629ED80E1801",
                    INIT_2B => X"940102A522C2D20022BEDC0239003800170103BF70015000700137001601B674",
                    INIT_2C => X"D600005F19205000D217005C1903006917201901012D19245000700062B6B500",
                    INIT_2D => X"1545154722D43B001A0114015000C560A6401000D5004BA01400500062C9F720",
                    INIT_2E => X"15681574157515411500155F152F152015541553154F15501500152F15201554",
                    INIT_2F => X"156315691573156115421520153A156E156F156915741561157A15691572156F",
                    INIT_30 => X"191D603703011000D10070006110700150007000FE03DDFF3E001D0170011500",
                    INIT_31 => X"20370069700067107001180C190103012069180C1901C069D2011701005C1800",
                    INIT_32 => X"180C02B5191470001C02144015001780180019077001005F1808192400691702",
                    INIT_33 => X"19077001005F18081924006917031800191D9000F700D65B005F1808192622C4",
                    INIT_34 => X"2069172219011000D200920322C4180C02B5191470001C02141E150017C01800",
                    INIT_35 => X"5000D2029205A34AF718D600005F180819262331D2022320D201005C1800191D",
                    INIT_36 => X"16007001004D1218004D0260004D0270B65464606570005F1808192867706660",
                    INIT_37 => X"0640700067706660655064407001B6746375D2079205D7107000677066601718",
                    INIT_38 => X"004D1200004D1260004D1200004D1200B634206917401901012D180C19282750",
                    INIT_39 => X"03B6142C132763B1F30ED280B31DB200003B0387130023B1B674004D52020230",
                    INIT_3A => X"B336D3C0B337D3A0B338D380B33903B61426131D03B61436133103B61431132C",
                    INIT_3B => X"12035000B67463B7C3401301E230004DA230B6345000B674003BB6F4003BD3E0",
                    INIT_3C => X"004C0270004C0280004C0290004C12032049004C0270004C0280004C0290004C",
                    INIT_3D => X"0280004C0290004C120203FA0D205000B67423D1130123D7D3FFE230004C1300",
                    INIT_3E => X"0280004C0290004C12D803FA500003FC63E5D20103F7004902D0004C0270004C",
                    INIT_3F => X"0000000020491204204912062049004C1205500063F3D20103F700490270004C",
                   INITP_00 => X"222222222AA0888A288888A30AAAD52020202677776376B77AAC00B56228A6AA",
                   INITP_01 => X"DD83777774DDD8282AA8274D08200A8228080A0888B4D82D62A2222222222222",
                   INITP_02 => X"DE0816075DDD55A022228035D5D75E0955602808A006088888888220B7777774",
                   INITP_03 => X"603603637A0034E0080B53553A008D55CB5D3600DE0556034D508776D75555FD",
                   INITP_04 => X"0EDDAB434E22C3360DD15623000578377EBB038282B35D5B42F55EDDE03B8203",
                   INITP_05 => X"AAAAAAAAAAAAAAAAA95D362D636208BD6DD56ED68888B6F58488A282EE8A3682",
                   INITP_06 => X"755EC2D438889605C358376083488C003820358223000E08ADC283483B77B15E",
                   INITP_07 => X"088A2CA2222ACA222222A7688888A2222B5A2AAA222208208350A2A08888A081")
      port map(  ADDRA => address_a(13 downto 0),
                   ENA => enable,
                  CLKA => clk,
                   DOA => data_out_a(31 downto 0),
                  DOPA => data_out_a(35 downto 32), 
                   DIA => data_in_a(31 downto 0),
                  DIPA => data_in_a(35 downto 32), 
                   WEA => "0000",
                REGCEA => '0',
                  RSTA => '0',
                 ADDRB => address_b(13 downto 0),
                   ENB => enable_b,
                  CLKB => clk_b,
                   DOB => data_out_b(31 downto 0),
                  DOPB => data_out_b(35 downto 32), 
                   DIB => data_in_b(31 downto 0),
                  DIPB => data_in_b(35 downto 32), 
                   WEB => we_b(3 downto 0),
                REGCEB => '0',
                  RSTB => '0');
    --               
    end generate s6;
    --
    --
    v6 : if (C_FAMILY = "V6") generate
      --
      address_a(13 downto 0) <= address(9 downto 0) & "1111";
      instruction <= data_out_a(17 downto 0);
      data_in_a(17 downto 0) <= "0000000000000000" & address(11 downto 10);
      jtag_dout <= data_out_b(17 downto 0);
      --
      no_loader : if (C_JTAG_LOADER_ENABLE = 0) generate
        data_in_b(17 downto 0) <= data_out_b(17 downto 0);
        address_b(13 downto 0) <= "11111111111111";
        we_b(3 downto 0) <= "0000";
        enable_b <= '0';
        rdl <= '0';
        clk_b <= '0';
      end generate no_loader;
      --
      loader : if (C_JTAG_LOADER_ENABLE = 1) generate
        data_in_b(17 downto 0) <= jtag_din(17 downto 0);
        address_b(13 downto 0) <= jtag_addr(9 downto 0) & "1111";
        we_b(3 downto 0) <= jtag_we & jtag_we & jtag_we & jtag_we;
        enable_b <= jtag_en(0);
        rdl <= rdl_bus(0);
        clk_b <= jtag_clk;
      end generate loader;
      -- 
      kcpsm6_rom: RAMB18E1
      generic map ( READ_WIDTH_A => 18,
                    WRITE_WIDTH_A => 18,
                    DOA_REG => 0,
                    INIT_A => "000000000000000000",
                    RSTREG_PRIORITY_A => "REGCE",
                    SRVAL_A => X"000000000000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    READ_WIDTH_B => 18,
                    WRITE_WIDTH_B => 18,
                    DOB_REG => 0,
                    INIT_B => X"000000000000000000",
                    RSTREG_PRIORITY_B => "REGCE",
                    SRVAL_B => X"000000000000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    RAM_MODE => "TDP",
                    RDADDR_COLLISION_HWCONFIG => "DELAYED_WRITE",
                    SIM_DEVICE => "VIRTEX6",
                    INIT_00 => X"D92000490820004C0920004C004C129FB001B03171FF003FB674003BB174003F",
                    INIT_01 => X"61107001039500A303C803EA601BD040900017001800190720132014F215F820",
                    INIT_02 => X"9511900095E9900015B9500035DFD000D57B9000D561202300E32013D1FF7000",
                    INIT_03 => X"129820431040110D120320431010112712005000150A500095F690001507E035",
                    INIT_04 => X"D2019205D208B6645000B674004C50006043B200B10090010000204310801196",
                    INIT_05 => X"38FB206B005338FB50000270004D0280004D0290004D1200B65450009204604E",
                    INIT_06 => X"1200B6545000B674004D00535804191F0069191E50000620006B0720004D0053",
                    INIT_07 => X"12FF004D1201004D1201004D12A8004D12C0004D1280004D1204004D1200004D",
                    INIT_08 => X"1202004D1201004D12DC004D1208004D1200004D1200004D12FF004D12FF004D",
                    INIT_09 => X"1601004DA2602110B674004D1202004D1201004D12A8004D12C0004D1203004D",
                    INIT_0A => X"004D1200004D1200B654606ED280B2005000D204005C180019395000609DC670",
                    INIT_0B => X"190CB654012DB615B71419040069B71318081900B674009D17131600004D1204",
                    INIT_0C => X"B726191D00691704191B012D16E81703191918000110B674009D171C16160053",
                    INIT_0D => X"1901003FB674003BB2742069003717101901500061101000D1FFB11C0069D701",
                    INIT_0E => X"D21820DFD21C182C2142D2172132D20020F9D214005C18281903206900371708",
                    INIT_0F => X"2308D2002350D217005C180819032137D21320D7D21D20D7D21B20D7D21A20D7",
                    INIT_10 => X"50002301D2152318D21320D7D21D20D7D21B20D7D21A20D7D21820DBD21C180C",
                    INIT_11 => X"006618C8006618A8006618880066186800661848170000661828006618081708",
                    INIT_12 => X"004D00535804165017001904710000691701190000D7182800D71808006618E8",
                    INIT_13 => X"7E007D007C007B007A0000691702182C1901203700691701182C1901206B0260",
                    INIT_14 => X"2183F700D60073006150FFFFDC0470006FF06CC07001005F1828192650007F00",
                    INIT_15 => X"1300004D1238004D0260004D0270B654005F182819280B700A601000F700D600",
                    INIT_16 => X"E5B0C4A02171D3FF2171D20A216CFFFFDC04350014011301E230004D14001500",
                    INIT_17 => X"4191DFFF7001006917401901012D182C192827900680B6740950084063306162",
                    INIT_18 => X"1000FB00DA0070006FF06CC06BB06AA06990688067706330700150007000DFFF",
                    INIT_19 => X"DD801FFE61A5F40AF50DD302B401B500500013001F014198DF0001DADF006BA0",
                    INIT_1A => X"61BEDD8061BF02D31AEC1B0269906880677061B060A050001A341B021C005000",
                    INIT_1B => X"70001D0021A1E1B41401170161BFC250A54021BED20003BF1907180017401415",
                    INIT_1C => X"10049000002B0025A5A01B0D1A07170050001E0070016990688067706B106A00",
                    INIT_1D => X"1C000D2003BF170018001907500061CBCAB01A01475061D09001490048004706",
                    INIT_1E => X"D262A1A101C850001A3E1B0217001800190661EDD220B205620502D31ADE1B02",
                    INIT_1F => X"D27250001A3E1B021C0561FFD27350001A3A1B021C0361F9D26521EA1C0261F3",
                    INIT_20 => X"700150001A0F1B021C04A1A101C861A102D31AE41B0250001A301B021C0361EA",
                    INIT_21 => X"6240A4E01E005000223E7000FE1FDDFF3E001D0170010037621DD30070006330",
                    INIT_22 => X"50001E001D0003FC221E221EDE0F222CCE301E01700039003800170103D97001",
                    INIT_23 => X"6CC07001700003FC03EA7001223E7000170018FE190670014BA01A311B02023E",
                    INIT_24 => X"1440151F1C00170018FF19066D706B806A907001005F18281924225EDC007000",
                    INIT_25 => X"18281924226BDC03226BDC041D006CC067D068B069A002C4182C02B519347000",
                    INIT_26 => X"D9010290700120DF182C02C4182C02B570001440151F70011934226DDC05005F",
                    INIT_27 => X"588018007001038770006380627FD902227E00D700DF627ED100180C01806278",
                    INIT_28 => X"700103B17000F80003B61300F702F800387F2290D700F80003B614091300F800",
                    INIT_29 => X"02A5A28079341800F20D9206F20C9207F20B005C1903F20A005C18081900F109",
                    INIT_2A => X"004D0280004D0290004D0260004D0270B6547000682022697000629ED80E1801",
                    INIT_2B => X"940102A522C2D20022BEDC0239003800170103BF70015000700137001601B674",
                    INIT_2C => X"D600005F19205000D217005C1903006917201901012D19245000700062B6B500",
                    INIT_2D => X"1545154722D43B001A0114015000C560A6401000D5004BA01400500062C9F720",
                    INIT_2E => X"15681574157515411500155F152F152015541553154F15501500152F15201554",
                    INIT_2F => X"156315691573156115421520153A156E156F156915741561157A15691572156F",
                    INIT_30 => X"191D603703011000D10070006110700150007000FE03DDFF3E001D0170011500",
                    INIT_31 => X"20370069700067107001180C190103012069180C1901C069D2011701005C1800",
                    INIT_32 => X"180C02B5191470001C02144015001780180019077001005F1808192400691702",
                    INIT_33 => X"19077001005F18081924006917031800191D9000F700D65B005F1808192622C4",
                    INIT_34 => X"2069172219011000D200920322C4180C02B5191470001C02141E150017C01800",
                    INIT_35 => X"5000D2029205A34AF718D600005F180819262331D2022320D201005C1800191D",
                    INIT_36 => X"16007001004D1218004D0260004D0270B65464606570005F1808192867706660",
                    INIT_37 => X"0640700067706660655064407001B6746375D2079205D7107000677066601718",
                    INIT_38 => X"004D1200004D1260004D1200004D1200B634206917401901012D180C19282750",
                    INIT_39 => X"03B6142C132763B1F30ED280B31DB200003B0387130023B1B674004D52020230",
                    INIT_3A => X"B336D3C0B337D3A0B338D380B33903B61426131D03B61436133103B61431132C",
                    INIT_3B => X"12035000B67463B7C3401301E230004DA230B6345000B674003BB6F4003BD3E0",
                    INIT_3C => X"004C0270004C0280004C0290004C12032049004C0270004C0280004C0290004C",
                    INIT_3D => X"0280004C0290004C120203FA0D205000B67423D1130123D7D3FFE230004C1300",
                    INIT_3E => X"0280004C0290004C12D803FA500003FC63E5D20103F7004902D0004C0270004C",
                    INIT_3F => X"0000000020491204204912062049004C1205500063F3D20103F700490270004C",
                   INITP_00 => X"222222222AA0888A288888A30AAAD52020202677776376B77AAC00B56228A6AA",
                   INITP_01 => X"DD83777774DDD8282AA8274D08200A8228080A0888B4D82D62A2222222222222",
                   INITP_02 => X"DE0816075DDD55A022228035D5D75E0955602808A006088888888220B7777774",
                   INITP_03 => X"603603637A0034E0080B53553A008D55CB5D3600DE0556034D508776D75555FD",
                   INITP_04 => X"0EDDAB434E22C3360DD15623000578377EBB038282B35D5B42F55EDDE03B8203",
                   INITP_05 => X"AAAAAAAAAAAAAAAAA95D362D636208BD6DD56ED68888B6F58488A282EE8A3682",
                   INITP_06 => X"755EC2D438889605C358376083488C003820358223000E08ADC283483B77B15E",
                   INITP_07 => X"088A2CA2222ACA222222A7688888A2222B5A2AAA222208208350A2A08888A081")
      port map(   ADDRARDADDR => address_a(13 downto 0),
                      ENARDEN => enable,
                    CLKARDCLK => clk,
                        DOADO => data_out_a(15 downto 0),
                      DOPADOP => data_out_a(17 downto 16), 
                        DIADI => data_in_a(15 downto 0),
                      DIPADIP => data_in_a(17 downto 16), 
                          WEA => "00",
                  REGCEAREGCE => '0',
                RSTRAMARSTRAM => '0',
                RSTREGARSTREG => '0',
                  ADDRBWRADDR => address_b(13 downto 0),
                      ENBWREN => enable_b,
                    CLKBWRCLK => clk_b,
                        DOBDO => data_out_b(15 downto 0),
                      DOPBDOP => data_out_b(17 downto 16), 
                        DIBDI => data_in_b(15 downto 0),
                      DIPBDIP => data_in_b(17 downto 16), 
                        WEBWE => we_b(3 downto 0),
                       REGCEB => '0',
                      RSTRAMB => '0',
                      RSTREGB => '0');
      --
    end generate v6;
    --
    --
    akv7 : if (C_FAMILY = "7S") generate
      --
      address_a(13 downto 0) <= address(9 downto 0) & "1111";
      instruction <= data_out_a(17 downto 0);
      data_in_a(17 downto 0) <= "0000000000000000" & address(11 downto 10);
      jtag_dout <= data_out_b(17 downto 0);
      --
      no_loader : if (C_JTAG_LOADER_ENABLE = 0) generate
        data_in_b(17 downto 0) <= data_out_b(17 downto 0);
        address_b(13 downto 0) <= "11111111111111";
        we_b(3 downto 0) <= "0000";
        enable_b <= '0';
        rdl <= '0';
        clk_b <= '0';
      end generate no_loader;
      --
      loader : if (C_JTAG_LOADER_ENABLE = 1) generate
        data_in_b(17 downto 0) <= jtag_din(17 downto 0);
        address_b(13 downto 0) <= jtag_addr(9 downto 0) & "1111";
        we_b(3 downto 0) <= jtag_we & jtag_we & jtag_we & jtag_we;
        enable_b <= jtag_en(0);
        rdl <= rdl_bus(0);
        clk_b <= jtag_clk;
      end generate loader;
      -- 
      kcpsm6_rom: RAMB18E1
      generic map ( READ_WIDTH_A => 18,
                    WRITE_WIDTH_A => 18,
                    DOA_REG => 0,
                    INIT_A => "000000000000000000",
                    RSTREG_PRIORITY_A => "REGCE",
                    SRVAL_A => X"000000000000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    READ_WIDTH_B => 18,
                    WRITE_WIDTH_B => 18,
                    DOB_REG => 0,
                    INIT_B => X"000000000000000000",
                    RSTREG_PRIORITY_B => "REGCE",
                    SRVAL_B => X"000000000000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    RAM_MODE => "TDP",
                    RDADDR_COLLISION_HWCONFIG => "DELAYED_WRITE",
                    SIM_DEVICE => "7SERIES",
                    INIT_00 => X"D92000490820004C0920004C004C129FB001B03171FF003FB674003BB174003F",
                    INIT_01 => X"61107001039500A303C803EA601BD040900017001800190720132014F215F820",
                    INIT_02 => X"9511900095E9900015B9500035DFD000D57B9000D561202300E32013D1FF7000",
                    INIT_03 => X"129820431040110D120320431010112712005000150A500095F690001507E035",
                    INIT_04 => X"D2019205D208B6645000B674004C50006043B200B10090010000204310801196",
                    INIT_05 => X"38FB206B005338FB50000270004D0280004D0290004D1200B65450009204604E",
                    INIT_06 => X"1200B6545000B674004D00535804191F0069191E50000620006B0720004D0053",
                    INIT_07 => X"12FF004D1201004D1201004D12A8004D12C0004D1280004D1204004D1200004D",
                    INIT_08 => X"1202004D1201004D12DC004D1208004D1200004D1200004D12FF004D12FF004D",
                    INIT_09 => X"1601004DA2602110B674004D1202004D1201004D12A8004D12C0004D1203004D",
                    INIT_0A => X"004D1200004D1200B654606ED280B2005000D204005C180019395000609DC670",
                    INIT_0B => X"190CB654012DB615B71419040069B71318081900B674009D17131600004D1204",
                    INIT_0C => X"B726191D00691704191B012D16E81703191918000110B674009D171C16160053",
                    INIT_0D => X"1901003FB674003BB2742069003717101901500061101000D1FFB11C0069D701",
                    INIT_0E => X"D21820DFD21C182C2142D2172132D20020F9D214005C18281903206900371708",
                    INIT_0F => X"2308D2002350D217005C180819032137D21320D7D21D20D7D21B20D7D21A20D7",
                    INIT_10 => X"50002301D2152318D21320D7D21D20D7D21B20D7D21A20D7D21820DBD21C180C",
                    INIT_11 => X"006618C8006618A8006618880066186800661848170000661828006618081708",
                    INIT_12 => X"004D00535804165017001904710000691701190000D7182800D71808006618E8",
                    INIT_13 => X"7E007D007C007B007A0000691702182C1901203700691701182C1901206B0260",
                    INIT_14 => X"2183F700D60073006150FFFFDC0470006FF06CC07001005F1828192650007F00",
                    INIT_15 => X"1300004D1238004D0260004D0270B654005F182819280B700A601000F700D600",
                    INIT_16 => X"E5B0C4A02171D3FF2171D20A216CFFFFDC04350014011301E230004D14001500",
                    INIT_17 => X"4191DFFF7001006917401901012D182C192827900680B6740950084063306162",
                    INIT_18 => X"1000FB00DA0070006FF06CC06BB06AA06990688067706330700150007000DFFF",
                    INIT_19 => X"DD801FFE61A5F40AF50DD302B401B500500013001F014198DF0001DADF006BA0",
                    INIT_1A => X"61BEDD8061BF02D31AEC1B0269906880677061B060A050001A341B021C005000",
                    INIT_1B => X"70001D0021A1E1B41401170161BFC250A54021BED20003BF1907180017401415",
                    INIT_1C => X"10049000002B0025A5A01B0D1A07170050001E0070016990688067706B106A00",
                    INIT_1D => X"1C000D2003BF170018001907500061CBCAB01A01475061D09001490048004706",
                    INIT_1E => X"D262A1A101C850001A3E1B0217001800190661EDD220B205620502D31ADE1B02",
                    INIT_1F => X"D27250001A3E1B021C0561FFD27350001A3A1B021C0361F9D26521EA1C0261F3",
                    INIT_20 => X"700150001A0F1B021C04A1A101C861A102D31AE41B0250001A301B021C0361EA",
                    INIT_21 => X"6240A4E01E005000223E7000FE1FDDFF3E001D0170010037621DD30070006330",
                    INIT_22 => X"50001E001D0003FC221E221EDE0F222CCE301E01700039003800170103D97001",
                    INIT_23 => X"6CC07001700003FC03EA7001223E7000170018FE190670014BA01A311B02023E",
                    INIT_24 => X"1440151F1C00170018FF19066D706B806A907001005F18281924225EDC007000",
                    INIT_25 => X"18281924226BDC03226BDC041D006CC067D068B069A002C4182C02B519347000",
                    INIT_26 => X"D9010290700120DF182C02C4182C02B570001440151F70011934226DDC05005F",
                    INIT_27 => X"588018007001038770006380627FD902227E00D700DF627ED100180C01806278",
                    INIT_28 => X"700103B17000F80003B61300F702F800387F2290D700F80003B614091300F800",
                    INIT_29 => X"02A5A28079341800F20D9206F20C9207F20B005C1903F20A005C18081900F109",
                    INIT_2A => X"004D0280004D0290004D0260004D0270B6547000682022697000629ED80E1801",
                    INIT_2B => X"940102A522C2D20022BEDC0239003800170103BF70015000700137001601B674",
                    INIT_2C => X"D600005F19205000D217005C1903006917201901012D19245000700062B6B500",
                    INIT_2D => X"1545154722D43B001A0114015000C560A6401000D5004BA01400500062C9F720",
                    INIT_2E => X"15681574157515411500155F152F152015541553154F15501500152F15201554",
                    INIT_2F => X"156315691573156115421520153A156E156F156915741561157A15691572156F",
                    INIT_30 => X"191D603703011000D10070006110700150007000FE03DDFF3E001D0170011500",
                    INIT_31 => X"20370069700067107001180C190103012069180C1901C069D2011701005C1800",
                    INIT_32 => X"180C02B5191470001C02144015001780180019077001005F1808192400691702",
                    INIT_33 => X"19077001005F18081924006917031800191D9000F700D65B005F1808192622C4",
                    INIT_34 => X"2069172219011000D200920322C4180C02B5191470001C02141E150017C01800",
                    INIT_35 => X"5000D2029205A34AF718D600005F180819262331D2022320D201005C1800191D",
                    INIT_36 => X"16007001004D1218004D0260004D0270B65464606570005F1808192867706660",
                    INIT_37 => X"0640700067706660655064407001B6746375D2079205D7107000677066601718",
                    INIT_38 => X"004D1200004D1260004D1200004D1200B634206917401901012D180C19282750",
                    INIT_39 => X"03B6142C132763B1F30ED280B31DB200003B0387130023B1B674004D52020230",
                    INIT_3A => X"B336D3C0B337D3A0B338D380B33903B61426131D03B61436133103B61431132C",
                    INIT_3B => X"12035000B67463B7C3401301E230004DA230B6345000B674003BB6F4003BD3E0",
                    INIT_3C => X"004C0270004C0280004C0290004C12032049004C0270004C0280004C0290004C",
                    INIT_3D => X"0280004C0290004C120203FA0D205000B67423D1130123D7D3FFE230004C1300",
                    INIT_3E => X"0280004C0290004C12D803FA500003FC63E5D20103F7004902D0004C0270004C",
                    INIT_3F => X"0000000020491204204912062049004C1205500063F3D20103F700490270004C",
                   INITP_00 => X"222222222AA0888A288888A30AAAD52020202677776376B77AAC00B56228A6AA",
                   INITP_01 => X"DD83777774DDD8282AA8274D08200A8228080A0888B4D82D62A2222222222222",
                   INITP_02 => X"DE0816075DDD55A022228035D5D75E0955602808A006088888888220B7777774",
                   INITP_03 => X"603603637A0034E0080B53553A008D55CB5D3600DE0556034D508776D75555FD",
                   INITP_04 => X"0EDDAB434E22C3360DD15623000578377EBB038282B35D5B42F55EDDE03B8203",
                   INITP_05 => X"AAAAAAAAAAAAAAAAA95D362D636208BD6DD56ED68888B6F58488A282EE8A3682",
                   INITP_06 => X"755EC2D438889605C358376083488C003820358223000E08ADC283483B77B15E",
                   INITP_07 => X"088A2CA2222ACA222222A7688888A2222B5A2AAA222208208350A2A08888A081")
      port map(   ADDRARDADDR => address_a(13 downto 0),
                      ENARDEN => enable,
                    CLKARDCLK => clk,
                        DOADO => data_out_a(15 downto 0),
                      DOPADOP => data_out_a(17 downto 16), 
                        DIADI => data_in_a(15 downto 0),
                      DIPADIP => data_in_a(17 downto 16), 
                          WEA => "00",
                  REGCEAREGCE => '0',
                RSTRAMARSTRAM => '0',
                RSTREGARSTREG => '0',
                  ADDRBWRADDR => address_b(13 downto 0),
                      ENBWREN => enable_b,
                    CLKBWRCLK => clk_b,
                        DOBDO => data_out_b(15 downto 0),
                      DOPBDOP => data_out_b(17 downto 16), 
                        DIBDI => data_in_b(15 downto 0),
                      DIPBDIP => data_in_b(17 downto 16), 
                        WEBWE => we_b(3 downto 0),
                       REGCEB => '0',
                      RSTRAMB => '0',
                      RSTREGB => '0');
      --
    end generate akv7;
    --
  end generate ram_1k_generate;
  --
  --
  --
  ram_2k_generate : if (C_RAM_SIZE_KWORDS = 2) generate
    --
    --
    s6: if (C_FAMILY = "S6") generate
      --
      address_a(13 downto 0) <= address(10 downto 0) & "000";
      instruction <= data_out_a_h(32) & data_out_a_h(7 downto 0) & data_out_a_l(32) & data_out_a_l(7 downto 0);
      data_in_a <= "00000000000000000000000000000000000" & address(11);
      jtag_dout <= data_out_b_h(32) & data_out_b_h(7 downto 0) & data_out_b_l(32) & data_out_b_l(7 downto 0);
      --
      no_loader : if (C_JTAG_LOADER_ENABLE = 0) generate
        data_in_b_l <= "000" & data_out_b_l(32) & "000000000000000000000000" & data_out_b_l(7 downto 0);
        data_in_b_h <= "000" & data_out_b_h(32) & "000000000000000000000000" & data_out_b_h(7 downto 0);
        address_b(13 downto 0) <= "00000000000000";
        we_b(3 downto 0) <= "0000";
        enable_b <= '0';
        rdl <= '0';
        clk_b <= '0';
      end generate no_loader;
      --
      loader : if (C_JTAG_LOADER_ENABLE = 1) generate
        data_in_b_h <= "000" & jtag_din(17) & "000000000000000000000000" & jtag_din(16 downto 9);
        data_in_b_l <= "000" & jtag_din(8) & "000000000000000000000000" & jtag_din(7 downto 0);
        address_b(13 downto 0) <= jtag_addr(10 downto 0) & "000";
        we_b(3 downto 0) <= jtag_we & jtag_we & jtag_we & jtag_we;
        enable_b <= jtag_en(0);
        rdl <= rdl_bus(0);
        clk_b <= jtag_clk;
      end generate loader;
      --
      kcpsm6_rom_l: RAMB16BWER
      generic map ( DATA_WIDTH_A => 9,
                    DOA_REG => 0,
                    EN_RSTRAM_A => FALSE,
                    INIT_A => X"000000000",
                    RST_PRIORITY_A => "CE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    DATA_WIDTH_B => 9,
                    DOB_REG => 0,
                    EN_RSTRAM_B => FALSE,
                    INIT_B => X"000000000",
                    RST_PRIORITY_B => "CE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    RSTTYPE => "SYNC",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    SIM_DEVICE => "SPARTAN6",
                    INIT_00 => X"100195A3C8EA1B4000000007131415202049204C204C4C9F0131FF3F743B743F",
                    INIT_01 => X"9843400D0343102700000A00F60007351100E900B900DF007B006123E313FF00",
                    INIT_02 => X"FB6B53FB00704D804D904D005400044E0105086400744C004300000100438096",
                    INIT_03 => X"FF4D014D014DA84DC04D804D044D004D005400744D53041F691E00206B204D53",
                    INIT_04 => X"014D6010744D024D014DA84DC04D034D024D014DDC4D084D004D004DFF4DFF4D",
                    INIT_05 => X"0C542D15140469130800749D13004D044D004D00546E800000045C0039009D70",
                    INIT_06 => X"013F743B7469371001001000FF1C6901261D69041B2DE803190010749D1C1653",
                    INIT_07 => X"080050175C08033713D71DD71BD71AD718DF1C2C42173200F9145C2803693708",
                    INIT_08 => X"66C866A86688666866480066286608080001151813D71DD71BD71AD718DB1C0C",
                    INIT_09 => X"000000000069022C013769012C016B604D530450000400690100D728D70866E8",
                    INIT_0A => X"004D384D604D70545F282870600000008300000050FF0400F0C0015F28260000",
                    INIT_0B => X"91FF016940012D2C2890807450403062B0A071FF710A6CFF04000101304D0000",
                    INIT_0C => X"80FEA50A0D0201000000019800DA00A000000000F0C0B0A090807030010000FF",
                    INIT_0D => X"0000A1B40101BF5040BE00BF07004015BE80BFD3EC02908070B0A00034020000",
                    INIT_0E => X"0020BF00000700CBB00150D00100000604002B25A00D07000000019080701000",
                    INIT_0F => X"72003E0205FF73003A0203F965EA02F362A1C8003E02000006ED200505D3DE02",
                    INIT_10 => X"40E000003E001FFF000101371D00003001000F0204A1C8A1D3E40200300203EA",
                    INIT_11 => X"C00100FCEA013E0000FE0601A031023E000000FC1E1E0F2C300100000001D901",
                    INIT_12 => X"28246B036B0400C0D0B0A0C42CB53400401F0000FF06708090015F28245E0000",
                    INIT_13 => X"8000018700807F027ED7DF7E000C8078019001DF2CC42CB500401F01346D055F",
                    INIT_14 => X"A58034000D060C070B5C030A5C08000901B10000B60002007F900000B6090000",
                    INIT_15 => X"01A5C200BE02000001BF0100010001744D804D904D604D7054002069009E0E01",
                    INIT_16 => X"4547D40001010060400000A00000C920005F2000175C036920012D240000B600",
                    INIT_17 => X"6369736142203A6E6F6974617A69726F68747541005F2F2054534F50002F2054",
                    INIT_18 => X"37690010010C0101690C016901015C001D37010000001001000003FF00010100",
                    INIT_19 => X"07015F08246903001D00005B5F0826C40CB51400024000800007015F08246902",
                    INIT_1A => X"0002054A18005F0826310220015C001D692201000003C40CB51400021E00C000",
                    INIT_1B => X"4000706050400174750705100070601800014D184D604D705460705F08287060",
                    INIT_1C => X"B62C27B10E801D003B8700B1744D02304D004D604D004D00346940012D0C2850",
                    INIT_1D => X"030074B74001304D303400743BF43BE036C037A0388039B6261DB63631B6312C",
                    INIT_1E => X"804C904C02FA200074D101D7FF304C004C704C804C904C03494C704C804C904C",
                    INIT_1F => X"000049044906494C0500F301F749704C804C904CD8FA00FCE501F749D04C704C",
                    INIT_20 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_21 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_22 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_23 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_24 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_25 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_26 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_27 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_28 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_29 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_30 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_31 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_32 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_33 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_34 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"A3000A0981ADDDA4AD480008100000000000014400000021112AAAA2AC508822",
                   INITP_01 => X"145566C175350D1676DAE6C4ED7F4AB1CECBBB518032DC8552940EC000215000",
                   INITP_02 => X"FFFFFFFFD13122F102C4000020234E2A150A802842A2570818AA301601451725",
                   INITP_03 => X"002805A0067900001C01FF6DBA70003B2895002618D160468AA223451324AA15",
                   INITP_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_07 => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(  ADDRA => address_a(13 downto 0),
                   ENA => enable,
                  CLKA => clk,
                   DOA => data_out_a_l(31 downto 0),
                  DOPA => data_out_a_l(35 downto 32), 
                   DIA => data_in_a(31 downto 0),
                  DIPA => data_in_a(35 downto 32), 
                   WEA => "0000",
                REGCEA => '0',
                  RSTA => '0',
                 ADDRB => address_b(13 downto 0),
                   ENB => enable_b,
                  CLKB => clk_b,
                   DOB => data_out_b_l(31 downto 0),
                  DOPB => data_out_b_l(35 downto 32), 
                   DIB => data_in_b_l(31 downto 0),
                  DIPB => data_in_b_l(35 downto 32), 
                   WEB => we_b(3 downto 0),
                REGCEB => '0',
                  RSTB => '0');
      -- 
      kcpsm6_rom_h: RAMB16BWER
      generic map ( DATA_WIDTH_A => 9,
                    DOA_REG => 0,
                    EN_RSTRAM_A => FALSE,
                    INIT_A => X"000000000",
                    RST_PRIORITY_A => "CE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    DATA_WIDTH_B => 9,
                    DOB_REG => 0,
                    EN_RSTRAM_B => FALSE,
                    INIT_B => X"000000000",
                    RST_PRIORITY_B => "CE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    RSTTYPE => "SYNC",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    SIM_DEVICE => "SPARTAN6",
                    INIT_00 => X"B0B801000101B068480B0C0C1090F9FCEC000400040000095858B8005B005800",
                    INIT_01 => X"091008080910080809288A28CAC88AF0CAC8CAC88A281AE8EAC8EA100090E8B8",
                    INIT_02 => X"1C10001C28010001000100095B2849B06949695B285B0028B0D9D8C800100808",
                    INIT_03 => X"09000900090009000900090009000900095B285B00002C0C000C280300030000",
                    INIT_04 => X"8B0051105B000900090009000900090009000900090009000900090009000900",
                    INIT_05 => X"0C5B005B5B0C005B0C0C5B000B0B0009000900095BB0E959A8E9000C0C28B0E3",
                    INIT_06 => X"0C005B005910000B0C28B088E85880EB5B0C000B0C000B0B0C0C005B000B0B00",
                    INIT_07 => X"91E991E9000C0C90E990E990E990E990E990E90C90E990E990E9000C0C10000B",
                    INIT_08 => X"000C000C000C000C000C0B000C000C0B2891E991E990E990E990E990E990E90C",
                    INIT_09 => X"BFBEBEBDBD000B0C0C10000B0C0C100100002C0B0B0CB8000B0C000C000C000C",
                    INIT_0A => X"090009000100015B000C0C050588FBEB90FBEBB9B0FFEEB8B7B6B8000C0C28BF",
                    INIT_0B => X"A0EFB8000B0C000C0C93835B0404B1B0F2E290E990E990FFEE9A8A8971000A0A",
                    INIT_0C => X"EE0FB0FAFAE95A5A28098FA0EF80EF3588FDEDB8B7B6B5B5B4B4B3B1B8A8B8EF",
                    INIT_0D => X"B80E10F08A8BB0E15290E9010C0C0B0AB0EEB0010D0DB4B4B3B0B0280D0D0EA8",
                    INIT_0E => X"0E06010B0C0C28B0E58D23B0C8A4A4A308C80000520D0D0B280FB8B4B4B3B5B5",
                    INIT_0F => X"E9280D0D0EB0E9280D0D0EB0E9100EB0E9D000280D0D0B0C0CB0E959B1010D0D",
                    INIT_10 => X"B1520F2891B8FFEE9F8EB800B1E9B8B1B8280D0D0ED000B0010D0D280D0D0EB0",
                    INIT_11 => X"B6B8B80101B811B80B0C0CB8250D0D01280F0E0111916F91E78FB89C9C8B01B8",
                    INIT_12 => X"0C0C91EE91EE0EB6B3B4B4010C010CB80A0A0E0B0C0CB6B5B5B8000C0C91EEB8",
                    INIT_13 => X"2C0CB801B8B1B1EC110000B1E80C00B1EC01B8100C010C01B80A0AB80C91EE00",
                    INIT_14 => X"0151BC0C7949794979000C79000C0C78B801B87C01097B7C1C91EB7C010A097C",
                    INIT_15 => X"CA0191E991EE9C9C8B01B828B89B8B5B00010001000100015BB8B411B8B1EC8C",
                    INIT_16 => X"0A0A119D8D8AA8E25388EA250A28B1FBEB000CA8E9000C000B0C000C28B8B1DA",
                    INIT_17 => X"0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A",
                    INIT_18 => X"1000B8B3B80C0C01100C0CE0E90B000C0CB00188E8B8B0B828B87FEE9F8EB80A",
                    INIT_19 => X"0CB8000C0C000B0C0CC8FBEB000C0C110C010CB80E0A0A0B0C0CB8000C0C000B",
                    INIT_1A => X"A86949D1FBEB000C0C91E991E9000C0C100B0C88E949110C010CB80E0A0A0B0C",
                    INIT_1B => X"83B8B3B3B2B2B85BB169496BB8B3B30B0BB80009000100015BB2B2000C0CB3B3",
                    INIT_1C => X"010A09B1F9E95959000109115B00290100090009000900095B100B0C000C0C93",
                    INIT_1D => X"09285BB1E1897100515B285B005B006959695969596959010A09010A09010A09",
                    INIT_1E => X"01000100090106285B118991E971000900010001000100091000010001000100",
                    INIT_1F => X"00001009100910000928B169010001000100010009012801B169010001000100",
                    INIT_20 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_21 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_22 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_23 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_24 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_25 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_26 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_27 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_28 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_29 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_30 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_31 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_32 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_33 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_34 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"A9554AA67E5224396232ACA65D55555555557CAB6AAD3F844455555D7E0C56DF",
                   INITP_01 => X"4515704C231070A0B250B0112095900EB2112A0C558489320462C12AAA94D554",
                   INITP_02 => X"FFFFFFFFE256552E6879AADC8AD9FB593AF13595281500657F199D231C3AC791",
                   INITP_03 => X"2B6D57B555D6AAD5737F552490DCAAC843986A90925492A064495032E99275C3",
                   INITP_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_07 => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(  ADDRA => address_a(13 downto 0),
                   ENA => enable,
                  CLKA => clk,
                   DOA => data_out_a_h(31 downto 0),
                  DOPA => data_out_a_h(35 downto 32), 
                   DIA => data_in_a(31 downto 0),
                  DIPA => data_in_a(35 downto 32), 
                   WEA => "0000",
                REGCEA => '0',
                  RSTA => '0',
                 ADDRB => address_b(13 downto 0),
                   ENB => enable_b,
                  CLKB => clk_b,
                   DOB => data_out_b_h(31 downto 0),
                  DOPB => data_out_b_h(35 downto 32), 
                   DIB => data_in_b_h(31 downto 0),
                  DIPB => data_in_b_h(35 downto 32), 
                   WEB => we_b(3 downto 0),
                REGCEB => '0',
                  RSTB => '0');
    --
    end generate s6;
    --
    --
    v6 : if (C_FAMILY = "V6") generate
      --
      address_a <= '1' & address(10 downto 0) & "1111";
      instruction <= data_out_a(33 downto 32) & data_out_a(15 downto 0);
      data_in_a <= "00000000000000000000000000000000000" & address(11);
      jtag_dout <= data_out_b(33 downto 32) & data_out_b(15 downto 0);
      --
      no_loader : if (C_JTAG_LOADER_ENABLE = 0) generate
        data_in_b <= "00" & data_out_b(33 downto 32) & "0000000000000000" & data_out_b(15 downto 0);
        address_b <= "1111111111111111";
        we_b <= "00000000";
        enable_b <= '0';
        rdl <= '0';
        clk_b <= '0';
      end generate no_loader;
      --
      loader : if (C_JTAG_LOADER_ENABLE = 1) generate
        data_in_b <= "00" & jtag_din(17 downto 16) & "0000000000000000" & jtag_din(15 downto 0);
        address_b <= '1' & jtag_addr(10 downto 0) & "1111";
        we_b <= jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we;
        enable_b <= jtag_en(0);
        rdl <= rdl_bus(0);
        clk_b <= jtag_clk;
      end generate loader;
      --
      kcpsm6_rom: RAMB36E1
      generic map ( READ_WIDTH_A => 18,
                    WRITE_WIDTH_A => 18,
                    DOA_REG => 0,
                    INIT_A => X"000000000",
                    RSTREG_PRIORITY_A => "REGCE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    READ_WIDTH_B => 18,
                    WRITE_WIDTH_B => 18,
                    DOB_REG => 0,
                    INIT_B => X"000000000",
                    RSTREG_PRIORITY_B => "REGCE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    RAM_MODE => "TDP",
                    RDADDR_COLLISION_HWCONFIG => "DELAYED_WRITE",
                    EN_ECC_READ => FALSE,
                    EN_ECC_WRITE => FALSE,
                    RAM_EXTENSION_A => "NONE",
                    RAM_EXTENSION_B => "NONE",
                    SIM_DEVICE => "VIRTEX6",
                    INIT_00 => X"D92000490820004C0920004C004C129FB001B03171FF003FB674003BB174003F",
                    INIT_01 => X"61107001039500A303C803EA601BD040900017001800190720132014F215F820",
                    INIT_02 => X"9511900095E9900015B9500035DFD000D57B9000D561202300E32013D1FF7000",
                    INIT_03 => X"129820431040110D120320431010112712005000150A500095F690001507E035",
                    INIT_04 => X"D2019205D208B6645000B674004C50006043B200B10090010000204310801196",
                    INIT_05 => X"38FB206B005338FB50000270004D0280004D0290004D1200B65450009204604E",
                    INIT_06 => X"1200B6545000B674004D00535804191F0069191E50000620006B0720004D0053",
                    INIT_07 => X"12FF004D1201004D1201004D12A8004D12C0004D1280004D1204004D1200004D",
                    INIT_08 => X"1202004D1201004D12DC004D1208004D1200004D1200004D12FF004D12FF004D",
                    INIT_09 => X"1601004DA2602110B674004D1202004D1201004D12A8004D12C0004D1203004D",
                    INIT_0A => X"004D1200004D1200B654606ED280B2005000D204005C180019395000609DC670",
                    INIT_0B => X"190CB654012DB615B71419040069B71318081900B674009D17131600004D1204",
                    INIT_0C => X"B726191D00691704191B012D16E81703191918000110B674009D171C16160053",
                    INIT_0D => X"1901003FB674003BB2742069003717101901500061101000D1FFB11C0069D701",
                    INIT_0E => X"D21820DFD21C182C2142D2172132D20020F9D214005C18281903206900371708",
                    INIT_0F => X"2308D2002350D217005C180819032137D21320D7D21D20D7D21B20D7D21A20D7",
                    INIT_10 => X"50002301D2152318D21320D7D21D20D7D21B20D7D21A20D7D21820DBD21C180C",
                    INIT_11 => X"006618C8006618A8006618880066186800661848170000661828006618081708",
                    INIT_12 => X"004D00535804165017001904710000691701190000D7182800D71808006618E8",
                    INIT_13 => X"7E007D007C007B007A0000691702182C1901203700691701182C1901206B0260",
                    INIT_14 => X"2183F700D60073006150FFFFDC0470006FF06CC07001005F1828192650007F00",
                    INIT_15 => X"1300004D1238004D0260004D0270B654005F182819280B700A601000F700D600",
                    INIT_16 => X"E5B0C4A02171D3FF2171D20A216CFFFFDC04350014011301E230004D14001500",
                    INIT_17 => X"4191DFFF7001006917401901012D182C192827900680B6740950084063306162",
                    INIT_18 => X"1000FB00DA0070006FF06CC06BB06AA06990688067706330700150007000DFFF",
                    INIT_19 => X"DD801FFE61A5F40AF50DD302B401B500500013001F014198DF0001DADF006BA0",
                    INIT_1A => X"61BEDD8061BF02D31AEC1B0269906880677061B060A050001A341B021C005000",
                    INIT_1B => X"70001D0021A1E1B41401170161BFC250A54021BED20003BF1907180017401415",
                    INIT_1C => X"10049000002B0025A5A01B0D1A07170050001E0070016990688067706B106A00",
                    INIT_1D => X"1C000D2003BF170018001907500061CBCAB01A01475061D09001490048004706",
                    INIT_1E => X"D262A1A101C850001A3E1B0217001800190661EDD220B205620502D31ADE1B02",
                    INIT_1F => X"D27250001A3E1B021C0561FFD27350001A3A1B021C0361F9D26521EA1C0261F3",
                    INIT_20 => X"700150001A0F1B021C04A1A101C861A102D31AE41B0250001A301B021C0361EA",
                    INIT_21 => X"6240A4E01E005000223E7000FE1FDDFF3E001D0170010037621DD30070006330",
                    INIT_22 => X"50001E001D0003FC221E221EDE0F222CCE301E01700039003800170103D97001",
                    INIT_23 => X"6CC07001700003FC03EA7001223E7000170018FE190670014BA01A311B02023E",
                    INIT_24 => X"1440151F1C00170018FF19066D706B806A907001005F18281924225EDC007000",
                    INIT_25 => X"18281924226BDC03226BDC041D006CC067D068B069A002C4182C02B519347000",
                    INIT_26 => X"D9010290700120DF182C02C4182C02B570001440151F70011934226DDC05005F",
                    INIT_27 => X"588018007001038770006380627FD902227E00D700DF627ED100180C01806278",
                    INIT_28 => X"700103B17000F80003B61300F702F800387F2290D700F80003B614091300F800",
                    INIT_29 => X"02A5A28079341800F20D9206F20C9207F20B005C1903F20A005C18081900F109",
                    INIT_2A => X"004D0280004D0290004D0260004D0270B6547000682022697000629ED80E1801",
                    INIT_2B => X"940102A522C2D20022BEDC0239003800170103BF70015000700137001601B674",
                    INIT_2C => X"D600005F19205000D217005C1903006917201901012D19245000700062B6B500",
                    INIT_2D => X"1545154722D43B001A0114015000C560A6401000D5004BA01400500062C9F720",
                    INIT_2E => X"15681574157515411500155F152F152015541553154F15501500152F15201554",
                    INIT_2F => X"156315691573156115421520153A156E156F156915741561157A15691572156F",
                    INIT_30 => X"191D603703011000D10070006110700150007000FE03DDFF3E001D0170011500",
                    INIT_31 => X"20370069700067107001180C190103012069180C1901C069D2011701005C1800",
                    INIT_32 => X"180C02B5191470001C02144015001780180019077001005F1808192400691702",
                    INIT_33 => X"19077001005F18081924006917031800191D9000F700D65B005F1808192622C4",
                    INIT_34 => X"2069172219011000D200920322C4180C02B5191470001C02141E150017C01800",
                    INIT_35 => X"5000D2029205A34AF718D600005F180819262331D2022320D201005C1800191D",
                    INIT_36 => X"16007001004D1218004D0260004D0270B65464606570005F1808192867706660",
                    INIT_37 => X"0640700067706660655064407001B6746375D2079205D7107000677066601718",
                    INIT_38 => X"004D1200004D1260004D1200004D1200B634206917401901012D180C19282750",
                    INIT_39 => X"03B6142C132763B1F30ED280B31DB200003B0387130023B1B674004D52020230",
                    INIT_3A => X"B336D3C0B337D3A0B338D380B33903B61426131D03B61436133103B61431132C",
                    INIT_3B => X"12035000B67463B7C3401301E230004DA230B6345000B674003BB6F4003BD3E0",
                    INIT_3C => X"004C0270004C0280004C0290004C12032049004C0270004C0280004C0290004C",
                    INIT_3D => X"0280004C0290004C120203FA0D205000B67423D1130123D7D3FFE230004C1300",
                    INIT_3E => X"0280004C0290004C12D803FA500003FC63E5D20103F7004902D0004C0270004C",
                    INIT_3F => X"0000000020491204204912062049004C1205500063F3D20103F700490270004C",
                    INIT_40 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_41 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_42 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_43 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_44 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_45 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_46 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_47 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_48 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_49 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_50 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_51 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_52 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_53 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_54 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_55 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_56 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_57 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_58 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_59 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_60 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_61 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_62 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_63 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_64 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_65 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_66 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_67 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_68 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_69 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_70 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_71 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_72 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_73 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_74 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_75 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_76 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_77 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_78 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_79 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"222222222AA0888A288888A30AAAD52020202677776376B77AAC00B56228A6AA",
                   INITP_01 => X"DD83777774DDD8282AA8274D08200A8228080A0888B4D82D62A2222222222222",
                   INITP_02 => X"DE0816075DDD55A022228035D5D75E0955602808A006088888888220B7777774",
                   INITP_03 => X"603603637A0034E0080B53553A008D55CB5D3600DE0556034D508776D75555FD",
                   INITP_04 => X"0EDDAB434E22C3360DD15623000578377EBB038282B35D5B42F55EDDE03B8203",
                   INITP_05 => X"AAAAAAAAAAAAAAAAA95D362D636208BD6DD56ED68888B6F58488A282EE8A3682",
                   INITP_06 => X"755EC2D438889605C358376083488C003820358223000E08ADC283483B77B15E",
                   INITP_07 => X"088A2CA2222ACA222222A7688888A2222B5A2AAA222208208350A2A08888A081",
                   INITP_08 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_09 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0A => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0B => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0C => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0D => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0E => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0F => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(   ADDRARDADDR => address_a,
                      ENARDEN => enable,
                    CLKARDCLK => clk,
                        DOADO => data_out_a(31 downto 0),
                      DOPADOP => data_out_a(35 downto 32), 
                        DIADI => data_in_a(31 downto 0),
                      DIPADIP => data_in_a(35 downto 32), 
                          WEA => "0000",
                  REGCEAREGCE => '0',
                RSTRAMARSTRAM => '0',
                RSTREGARSTREG => '0',
                  ADDRBWRADDR => address_b,
                      ENBWREN => enable_b,
                    CLKBWRCLK => clk_b,
                        DOBDO => data_out_b(31 downto 0),
                      DOPBDOP => data_out_b(35 downto 32), 
                        DIBDI => data_in_b(31 downto 0),
                      DIPBDIP => data_in_b(35 downto 32), 
                        WEBWE => we_b,
                       REGCEB => '0',
                      RSTRAMB => '0',
                      RSTREGB => '0',
                   CASCADEINA => '0',
                   CASCADEINB => '0',
                INJECTDBITERR => '0',
                INJECTSBITERR => '0');
      --
    end generate v6;
    --
    --
    akv7 : if (C_FAMILY = "7S") generate
      --
      address_a <= '1' & address(10 downto 0) & "1111";
      instruction <= data_out_a(33 downto 32) & data_out_a(15 downto 0);
      data_in_a <= "00000000000000000000000000000000000" & address(11);
      jtag_dout <= data_out_b(33 downto 32) & data_out_b(15 downto 0);
      --
      no_loader : if (C_JTAG_LOADER_ENABLE = 0) generate
        data_in_b <= "00" & data_out_b(33 downto 32) & "0000000000000000" & data_out_b(15 downto 0);
        address_b <= "1111111111111111";
        we_b <= "00000000";
        enable_b <= '0';
        rdl <= '0';
        clk_b <= '0';
      end generate no_loader;
      --
      loader : if (C_JTAG_LOADER_ENABLE = 1) generate
        data_in_b <= "00" & jtag_din(17 downto 16) & "0000000000000000" & jtag_din(15 downto 0);
        address_b <= '1' & jtag_addr(10 downto 0) & "1111";
        we_b <= jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we;
        enable_b <= jtag_en(0);
        rdl <= rdl_bus(0);
        clk_b <= jtag_clk;
      end generate loader;
      --
      kcpsm6_rom: RAMB36E1
      generic map ( READ_WIDTH_A => 18,
                    WRITE_WIDTH_A => 18,
                    DOA_REG => 0,
                    INIT_A => X"000000000",
                    RSTREG_PRIORITY_A => "REGCE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    READ_WIDTH_B => 18,
                    WRITE_WIDTH_B => 18,
                    DOB_REG => 0,
                    INIT_B => X"000000000",
                    RSTREG_PRIORITY_B => "REGCE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    RAM_MODE => "TDP",
                    RDADDR_COLLISION_HWCONFIG => "DELAYED_WRITE",
                    EN_ECC_READ => FALSE,
                    EN_ECC_WRITE => FALSE,
                    RAM_EXTENSION_A => "NONE",
                    RAM_EXTENSION_B => "NONE",
                    SIM_DEVICE => "7SERIES",
                    INIT_00 => X"D92000490820004C0920004C004C129FB001B03171FF003FB674003BB174003F",
                    INIT_01 => X"61107001039500A303C803EA601BD040900017001800190720132014F215F820",
                    INIT_02 => X"9511900095E9900015B9500035DFD000D57B9000D561202300E32013D1FF7000",
                    INIT_03 => X"129820431040110D120320431010112712005000150A500095F690001507E035",
                    INIT_04 => X"D2019205D208B6645000B674004C50006043B200B10090010000204310801196",
                    INIT_05 => X"38FB206B005338FB50000270004D0280004D0290004D1200B65450009204604E",
                    INIT_06 => X"1200B6545000B674004D00535804191F0069191E50000620006B0720004D0053",
                    INIT_07 => X"12FF004D1201004D1201004D12A8004D12C0004D1280004D1204004D1200004D",
                    INIT_08 => X"1202004D1201004D12DC004D1208004D1200004D1200004D12FF004D12FF004D",
                    INIT_09 => X"1601004DA2602110B674004D1202004D1201004D12A8004D12C0004D1203004D",
                    INIT_0A => X"004D1200004D1200B654606ED280B2005000D204005C180019395000609DC670",
                    INIT_0B => X"190CB654012DB615B71419040069B71318081900B674009D17131600004D1204",
                    INIT_0C => X"B726191D00691704191B012D16E81703191918000110B674009D171C16160053",
                    INIT_0D => X"1901003FB674003BB2742069003717101901500061101000D1FFB11C0069D701",
                    INIT_0E => X"D21820DFD21C182C2142D2172132D20020F9D214005C18281903206900371708",
                    INIT_0F => X"2308D2002350D217005C180819032137D21320D7D21D20D7D21B20D7D21A20D7",
                    INIT_10 => X"50002301D2152318D21320D7D21D20D7D21B20D7D21A20D7D21820DBD21C180C",
                    INIT_11 => X"006618C8006618A8006618880066186800661848170000661828006618081708",
                    INIT_12 => X"004D00535804165017001904710000691701190000D7182800D71808006618E8",
                    INIT_13 => X"7E007D007C007B007A0000691702182C1901203700691701182C1901206B0260",
                    INIT_14 => X"2183F700D60073006150FFFFDC0470006FF06CC07001005F1828192650007F00",
                    INIT_15 => X"1300004D1238004D0260004D0270B654005F182819280B700A601000F700D600",
                    INIT_16 => X"E5B0C4A02171D3FF2171D20A216CFFFFDC04350014011301E230004D14001500",
                    INIT_17 => X"4191DFFF7001006917401901012D182C192827900680B6740950084063306162",
                    INIT_18 => X"1000FB00DA0070006FF06CC06BB06AA06990688067706330700150007000DFFF",
                    INIT_19 => X"DD801FFE61A5F40AF50DD302B401B500500013001F014198DF0001DADF006BA0",
                    INIT_1A => X"61BEDD8061BF02D31AEC1B0269906880677061B060A050001A341B021C005000",
                    INIT_1B => X"70001D0021A1E1B41401170161BFC250A54021BED20003BF1907180017401415",
                    INIT_1C => X"10049000002B0025A5A01B0D1A07170050001E0070016990688067706B106A00",
                    INIT_1D => X"1C000D2003BF170018001907500061CBCAB01A01475061D09001490048004706",
                    INIT_1E => X"D262A1A101C850001A3E1B0217001800190661EDD220B205620502D31ADE1B02",
                    INIT_1F => X"D27250001A3E1B021C0561FFD27350001A3A1B021C0361F9D26521EA1C0261F3",
                    INIT_20 => X"700150001A0F1B021C04A1A101C861A102D31AE41B0250001A301B021C0361EA",
                    INIT_21 => X"6240A4E01E005000223E7000FE1FDDFF3E001D0170010037621DD30070006330",
                    INIT_22 => X"50001E001D0003FC221E221EDE0F222CCE301E01700039003800170103D97001",
                    INIT_23 => X"6CC07001700003FC03EA7001223E7000170018FE190670014BA01A311B02023E",
                    INIT_24 => X"1440151F1C00170018FF19066D706B806A907001005F18281924225EDC007000",
                    INIT_25 => X"18281924226BDC03226BDC041D006CC067D068B069A002C4182C02B519347000",
                    INIT_26 => X"D9010290700120DF182C02C4182C02B570001440151F70011934226DDC05005F",
                    INIT_27 => X"588018007001038770006380627FD902227E00D700DF627ED100180C01806278",
                    INIT_28 => X"700103B17000F80003B61300F702F800387F2290D700F80003B614091300F800",
                    INIT_29 => X"02A5A28079341800F20D9206F20C9207F20B005C1903F20A005C18081900F109",
                    INIT_2A => X"004D0280004D0290004D0260004D0270B6547000682022697000629ED80E1801",
                    INIT_2B => X"940102A522C2D20022BEDC0239003800170103BF70015000700137001601B674",
                    INIT_2C => X"D600005F19205000D217005C1903006917201901012D19245000700062B6B500",
                    INIT_2D => X"1545154722D43B001A0114015000C560A6401000D5004BA01400500062C9F720",
                    INIT_2E => X"15681574157515411500155F152F152015541553154F15501500152F15201554",
                    INIT_2F => X"156315691573156115421520153A156E156F156915741561157A15691572156F",
                    INIT_30 => X"191D603703011000D10070006110700150007000FE03DDFF3E001D0170011500",
                    INIT_31 => X"20370069700067107001180C190103012069180C1901C069D2011701005C1800",
                    INIT_32 => X"180C02B5191470001C02144015001780180019077001005F1808192400691702",
                    INIT_33 => X"19077001005F18081924006917031800191D9000F700D65B005F1808192622C4",
                    INIT_34 => X"2069172219011000D200920322C4180C02B5191470001C02141E150017C01800",
                    INIT_35 => X"5000D2029205A34AF718D600005F180819262331D2022320D201005C1800191D",
                    INIT_36 => X"16007001004D1218004D0260004D0270B65464606570005F1808192867706660",
                    INIT_37 => X"0640700067706660655064407001B6746375D2079205D7107000677066601718",
                    INIT_38 => X"004D1200004D1260004D1200004D1200B634206917401901012D180C19282750",
                    INIT_39 => X"03B6142C132763B1F30ED280B31DB200003B0387130023B1B674004D52020230",
                    INIT_3A => X"B336D3C0B337D3A0B338D380B33903B61426131D03B61436133103B61431132C",
                    INIT_3B => X"12035000B67463B7C3401301E230004DA230B6345000B674003BB6F4003BD3E0",
                    INIT_3C => X"004C0270004C0280004C0290004C12032049004C0270004C0280004C0290004C",
                    INIT_3D => X"0280004C0290004C120203FA0D205000B67423D1130123D7D3FFE230004C1300",
                    INIT_3E => X"0280004C0290004C12D803FA500003FC63E5D20103F7004902D0004C0270004C",
                    INIT_3F => X"0000000020491204204912062049004C1205500063F3D20103F700490270004C",
                    INIT_40 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_41 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_42 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_43 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_44 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_45 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_46 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_47 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_48 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_49 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_50 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_51 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_52 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_53 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_54 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_55 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_56 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_57 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_58 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_59 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_60 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_61 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_62 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_63 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_64 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_65 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_66 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_67 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_68 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_69 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_70 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_71 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_72 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_73 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_74 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_75 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_76 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_77 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_78 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_79 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"222222222AA0888A288888A30AAAD52020202677776376B77AAC00B56228A6AA",
                   INITP_01 => X"DD83777774DDD8282AA8274D08200A8228080A0888B4D82D62A2222222222222",
                   INITP_02 => X"DE0816075DDD55A022228035D5D75E0955602808A006088888888220B7777774",
                   INITP_03 => X"603603637A0034E0080B53553A008D55CB5D3600DE0556034D508776D75555FD",
                   INITP_04 => X"0EDDAB434E22C3360DD15623000578377EBB038282B35D5B42F55EDDE03B8203",
                   INITP_05 => X"AAAAAAAAAAAAAAAAA95D362D636208BD6DD56ED68888B6F58488A282EE8A3682",
                   INITP_06 => X"755EC2D438889605C358376083488C003820358223000E08ADC283483B77B15E",
                   INITP_07 => X"088A2CA2222ACA222222A7688888A2222B5A2AAA222208208350A2A08888A081",
                   INITP_08 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_09 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0A => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0B => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0C => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0D => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0E => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0F => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(   ADDRARDADDR => address_a,
                      ENARDEN => enable,
                    CLKARDCLK => clk,
                        DOADO => data_out_a(31 downto 0),
                      DOPADOP => data_out_a(35 downto 32), 
                        DIADI => data_in_a(31 downto 0),
                      DIPADIP => data_in_a(35 downto 32), 
                          WEA => "0000",
                  REGCEAREGCE => '0',
                RSTRAMARSTRAM => '0',
                RSTREGARSTREG => '0',
                  ADDRBWRADDR => address_b,
                      ENBWREN => enable_b,
                    CLKBWRCLK => clk_b,
                        DOBDO => data_out_b(31 downto 0),
                      DOPBDOP => data_out_b(35 downto 32), 
                        DIBDI => data_in_b(31 downto 0),
                      DIPBDIP => data_in_b(35 downto 32), 
                        WEBWE => we_b,
                       REGCEB => '0',
                      RSTRAMB => '0',
                      RSTREGB => '0',
                   CASCADEINA => '0',
                   CASCADEINB => '0',
                INJECTDBITERR => '0',
                INJECTSBITERR => '0');
      --
    end generate akv7;
    --
  end generate ram_2k_generate;
  --
  --	
  ram_4k_generate : if (C_RAM_SIZE_KWORDS = 4) generate
    s6: if (C_FAMILY = "S6") generate
      --
      address_a(13 downto 0) <= address(10 downto 0) & "000";
      data_in_a <= "000000000000000000000000000000000000";
      --
      s6_a11_flop: FD
      port map (  D => address(11),
                  Q => pipe_a11,
                  C => clk);
      --
      s6_4k_mux0_lut: LUT6_2
      generic map (INIT => X"FF00F0F0CCCCAAAA")
      port map( I0 => data_out_a_ll(0),
                I1 => data_out_a_hl(0),
                I2 => data_out_a_ll(1),
                I3 => data_out_a_hl(1),
                I4 => pipe_a11,
                I5 => '1',
                O5 => instruction(0),
                O6 => instruction(1));
      --
      s6_4k_mux2_lut: LUT6_2
      generic map (INIT => X"FF00F0F0CCCCAAAA")
      port map( I0 => data_out_a_ll(2),
                I1 => data_out_a_hl(2),
                I2 => data_out_a_ll(3),
                I3 => data_out_a_hl(3),
                I4 => pipe_a11,
                I5 => '1',
                O5 => instruction(2),
                O6 => instruction(3));
      --
      s6_4k_mux4_lut: LUT6_2
      generic map (INIT => X"FF00F0F0CCCCAAAA")
      port map( I0 => data_out_a_ll(4),
                I1 => data_out_a_hl(4),
                I2 => data_out_a_ll(5),
                I3 => data_out_a_hl(5),
                I4 => pipe_a11,
                I5 => '1',
                O5 => instruction(4),
                O6 => instruction(5));
      --
      s6_4k_mux6_lut: LUT6_2
      generic map (INIT => X"FF00F0F0CCCCAAAA")
      port map( I0 => data_out_a_ll(6),
                I1 => data_out_a_hl(6),
                I2 => data_out_a_ll(7),
                I3 => data_out_a_hl(7),
                I4 => pipe_a11,
                I5 => '1',
                O5 => instruction(6),
                O6 => instruction(7));
      --
      s6_4k_mux8_lut: LUT6_2
      generic map (INIT => X"FF00F0F0CCCCAAAA")
      port map( I0 => data_out_a_ll(32),
                I1 => data_out_a_hl(32),
                I2 => data_out_a_lh(0),
                I3 => data_out_a_hh(0),
                I4 => pipe_a11,
                I5 => '1',
                O5 => instruction(8),
                O6 => instruction(9));
      --
      s6_4k_mux10_lut: LUT6_2
      generic map (INIT => X"FF00F0F0CCCCAAAA")
      port map( I0 => data_out_a_lh(1),
                I1 => data_out_a_hh(1),
                I2 => data_out_a_lh(2),
                I3 => data_out_a_hh(2),
                I4 => pipe_a11,
                I5 => '1',
                O5 => instruction(10),
                O6 => instruction(11));
      --
      s6_4k_mux12_lut: LUT6_2
      generic map (INIT => X"FF00F0F0CCCCAAAA")
      port map( I0 => data_out_a_lh(3),
                I1 => data_out_a_hh(3),
                I2 => data_out_a_lh(4),
                I3 => data_out_a_hh(4),
                I4 => pipe_a11,
                I5 => '1',
                O5 => instruction(12),
                O6 => instruction(13));
      --
      s6_4k_mux14_lut: LUT6_2
      generic map (INIT => X"FF00F0F0CCCCAAAA")
      port map( I0 => data_out_a_lh(5),
                I1 => data_out_a_hh(5),
                I2 => data_out_a_lh(6),
                I3 => data_out_a_hh(6),
                I4 => pipe_a11,
                I5 => '1',
                O5 => instruction(14),
                O6 => instruction(15));
      --
      s6_4k_mux16_lut: LUT6_2
      generic map (INIT => X"FF00F0F0CCCCAAAA")
      port map( I0 => data_out_a_lh(7),
                I1 => data_out_a_hh(7),
                I2 => data_out_a_lh(32),
                I3 => data_out_a_hh(32),
                I4 => pipe_a11,
                I5 => '1',
                O5 => instruction(16),
                O6 => instruction(17));
      --
      no_loader : if (C_JTAG_LOADER_ENABLE = 0) generate
        data_in_b_ll <= "000" & data_out_b_ll(32) & "000000000000000000000000" & data_out_b_ll(7 downto 0);
        data_in_b_lh <= "000" & data_out_b_lh(32) & "000000000000000000000000" & data_out_b_lh(7 downto 0);
        data_in_b_hl <= "000" & data_out_b_hl(32) & "000000000000000000000000" & data_out_b_hl(7 downto 0);
        data_in_b_hh <= "000" & data_out_b_hh(32) & "000000000000000000000000" & data_out_b_hh(7 downto 0);
        address_b(13 downto 0) <= "00000000000000";
        we_b_l(3 downto 0) <= "0000";
        we_b_h(3 downto 0) <= "0000";
        enable_b <= '0';
        rdl <= '0';
        clk_b <= '0';
        jtag_dout <= data_out_b_lh(32) & data_out_b_lh(7 downto 0) & data_out_b_ll(32) & data_out_b_ll(7 downto 0);
      end generate no_loader;
      --
      loader : if (C_JTAG_LOADER_ENABLE = 1) generate
        data_in_b_lh <= "000" & jtag_din(17) & "000000000000000000000000" & jtag_din(16 downto 9);
        data_in_b_ll <= "000" & jtag_din(8) & "000000000000000000000000" & jtag_din(7 downto 0);
        data_in_b_hh <= "000" & jtag_din(17) & "000000000000000000000000" & jtag_din(16 downto 9);
        data_in_b_hl <= "000" & jtag_din(8) & "000000000000000000000000" & jtag_din(7 downto 0);
        address_b(13 downto 0) <= jtag_addr(10 downto 0) & "000";
        --
        s6_4k_jtag_we_lut: LUT6_2
        generic map (INIT => X"8000000020000000")
        port map( I0 => jtag_we,
                  I1 => jtag_addr(11),
                  I2 => '1',
                  I3 => '1',
                  I4 => '1',
                  I5 => '1',
                  O5 => jtag_we_l,
                  O6 => jtag_we_h);
        --
        we_b_l(3 downto 0) <= jtag_we_l & jtag_we_l & jtag_we_l & jtag_we_l;
        we_b_h(3 downto 0) <= jtag_we_h & jtag_we_h & jtag_we_h & jtag_we_h;
        --
        enable_b <= jtag_en(0);
        rdl <= rdl_bus(0);
        clk_b <= jtag_clk;
        --
        s6_4k_jtag_mux0_lut: LUT6_2
        generic map (INIT => X"FF00F0F0CCCCAAAA")
        port map( I0 => data_out_b_ll(0),
                  I1 => data_out_b_hl(0),
                  I2 => data_out_b_ll(1),
                  I3 => data_out_b_hl(1),
                  I4 => jtag_addr(11),
                  I5 => '1',
                  O5 => jtag_dout(0),
                  O6 => jtag_dout(1));
        --
        s6_4k_jtag_mux2_lut: LUT6_2
        generic map (INIT => X"FF00F0F0CCCCAAAA")
        port map( I0 => data_out_b_ll(2),
                  I1 => data_out_b_hl(2),
                  I2 => data_out_b_ll(3),
                  I3 => data_out_b_hl(3),
                  I4 => jtag_addr(11),
                  I5 => '1',
                  O5 => jtag_dout(2),
                  O6 => jtag_dout(3));
        --
        s6_4k_jtag_mux4_lut: LUT6_2
        generic map (INIT => X"FF00F0F0CCCCAAAA")
        port map( I0 => data_out_b_ll(4),
                  I1 => data_out_b_hl(4),
                  I2 => data_out_b_ll(5),
                  I3 => data_out_b_hl(5),
                  I4 => jtag_addr(11),
                  I5 => '1',
                  O5 => jtag_dout(4),
                  O6 => jtag_dout(5));
        --
        s6_4k_jtag_mux6_lut: LUT6_2
        generic map (INIT => X"FF00F0F0CCCCAAAA")
        port map( I0 => data_out_b_ll(6),
                  I1 => data_out_b_hl(6),
                  I2 => data_out_b_ll(7),
                  I3 => data_out_b_hl(7),
                  I4 => jtag_addr(11),
                  I5 => '1',
                  O5 => jtag_dout(6),
                  O6 => jtag_dout(7));
        --
        s6_4k_jtag_mux8_lut: LUT6_2
        generic map (INIT => X"FF00F0F0CCCCAAAA")
        port map( I0 => data_out_b_ll(32),
                  I1 => data_out_b_hl(32),
                  I2 => data_out_b_lh(0),
                  I3 => data_out_b_hh(0),
                  I4 => jtag_addr(11),
                  I5 => '1',
                  O5 => jtag_dout(8),
                  O6 => jtag_dout(9));
        --
        s6_4k_jtag_mux10_lut: LUT6_2
        generic map (INIT => X"FF00F0F0CCCCAAAA")
        port map( I0 => data_out_b_lh(1),
                  I1 => data_out_b_hh(1),
                  I2 => data_out_b_lh(2),
                  I3 => data_out_b_hh(2),
                  I4 => jtag_addr(11),
                  I5 => '1',
                  O5 => jtag_dout(10),
                  O6 => jtag_dout(11));
        --
        s6_4k_jtag_mux12_lut: LUT6_2
        generic map (INIT => X"FF00F0F0CCCCAAAA")
        port map( I0 => data_out_b_lh(3),
                  I1 => data_out_b_hh(3),
                  I2 => data_out_b_lh(4),
                  I3 => data_out_b_hh(4),
                  I4 => jtag_addr(11),
                  I5 => '1',
                  O5 => jtag_dout(12),
                  O6 => jtag_dout(13));
        --
        s6_4k_jtag_mux14_lut: LUT6_2
        generic map (INIT => X"FF00F0F0CCCCAAAA")
        port map( I0 => data_out_b_lh(5),
                  I1 => data_out_b_hh(5),
                  I2 => data_out_b_lh(6),
                  I3 => data_out_b_hh(6),
                  I4 => jtag_addr(11),
                  I5 => '1',
                  O5 => jtag_dout(14),
                  O6 => jtag_dout(15));
        --
        s6_4k_jtag_mux16_lut: LUT6_2
        generic map (INIT => X"FF00F0F0CCCCAAAA")
        port map( I0 => data_out_b_lh(7),
                  I1 => data_out_b_hh(7),
                  I2 => data_out_b_lh(32),
                  I3 => data_out_b_hh(32),
                  I4 => jtag_addr(11),
                  I5 => '1',
                  O5 => jtag_dout(16),
                  O6 => jtag_dout(17));
      --
      end generate loader;
      --
      kcpsm6_rom_ll: RAMB16BWER
      generic map ( DATA_WIDTH_A => 9,
                    DOA_REG => 0,
                    EN_RSTRAM_A => FALSE,
                    INIT_A => X"000000000",
                    RST_PRIORITY_A => "CE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    DATA_WIDTH_B => 9,
                    DOB_REG => 0,
                    EN_RSTRAM_B => FALSE,
                    INIT_B => X"000000000",
                    RST_PRIORITY_B => "CE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    RSTTYPE => "SYNC",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    SIM_DEVICE => "SPARTAN6",
                    INIT_00 => X"100195A3C8EA1B4000000007131415202049204C204C4C9F0131FF3F743B743F",
                    INIT_01 => X"9843400D0343102700000A00F60007351100E900B900DF007B006123E313FF00",
                    INIT_02 => X"FB6B53FB00704D804D904D005400044E0105086400744C004300000100438096",
                    INIT_03 => X"FF4D014D014DA84DC04D804D044D004D005400744D53041F691E00206B204D53",
                    INIT_04 => X"014D6010744D024D014DA84DC04D034D024D014DDC4D084D004D004DFF4DFF4D",
                    INIT_05 => X"0C542D15140469130800749D13004D044D004D00546E800000045C0039009D70",
                    INIT_06 => X"013F743B7469371001001000FF1C6901261D69041B2DE803190010749D1C1653",
                    INIT_07 => X"080050175C08033713D71DD71BD71AD718DF1C2C42173200F9145C2803693708",
                    INIT_08 => X"66C866A86688666866480066286608080001151813D71DD71BD71AD718DB1C0C",
                    INIT_09 => X"000000000069022C013769012C016B604D530450000400690100D728D70866E8",
                    INIT_0A => X"004D384D604D70545F282870600000008300000050FF0400F0C0015F28260000",
                    INIT_0B => X"91FF016940012D2C2890807450403062B0A071FF710A6CFF04000101304D0000",
                    INIT_0C => X"80FEA50A0D0201000000019800DA00A000000000F0C0B0A090807030010000FF",
                    INIT_0D => X"0000A1B40101BF5040BE00BF07004015BE80BFD3EC02908070B0A00034020000",
                    INIT_0E => X"0020BF00000700CBB00150D00100000604002B25A00D07000000019080701000",
                    INIT_0F => X"72003E0205FF73003A0203F965EA02F362A1C8003E02000006ED200505D3DE02",
                    INIT_10 => X"40E000003E001FFF000101371D00003001000F0204A1C8A1D3E40200300203EA",
                    INIT_11 => X"C00100FCEA013E0000FE0601A031023E000000FC1E1E0F2C300100000001D901",
                    INIT_12 => X"28246B036B0400C0D0B0A0C42CB53400401F0000FF06708090015F28245E0000",
                    INIT_13 => X"8000018700807F027ED7DF7E000C8078019001DF2CC42CB500401F01346D055F",
                    INIT_14 => X"A58034000D060C070B5C030A5C08000901B10000B60002007F900000B6090000",
                    INIT_15 => X"01A5C200BE02000001BF0100010001744D804D904D604D7054002069009E0E01",
                    INIT_16 => X"4547D40001010060400000A00000C920005F2000175C036920012D240000B600",
                    INIT_17 => X"6369736142203A6E6F6974617A69726F68747541005F2F2054534F50002F2054",
                    INIT_18 => X"37690010010C0101690C016901015C001D37010000001001000003FF00010100",
                    INIT_19 => X"07015F08246903001D00005B5F0826C40CB51400024000800007015F08246902",
                    INIT_1A => X"0002054A18005F0826310220015C001D692201000003C40CB51400021E00C000",
                    INIT_1B => X"4000706050400174750705100070601800014D184D604D705460705F08287060",
                    INIT_1C => X"B62C27B10E801D003B8700B1744D02304D004D604D004D00346940012D0C2850",
                    INIT_1D => X"030074B74001304D303400743BF43BE036C037A0388039B6261DB63631B6312C",
                    INIT_1E => X"804C904C02FA200074D101D7FF304C004C704C804C904C03494C704C804C904C",
                    INIT_1F => X"000049044906494C0500F301F749704C804C904CD8FA00FCE501F749D04C704C",
                    INIT_20 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_21 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_22 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_23 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_24 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_25 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_26 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_27 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_28 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_29 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_30 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_31 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_32 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_33 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_34 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"A3000A0981ADDDA4AD480008100000000000014400000021112AAAA2AC508822",
                   INITP_01 => X"145566C175350D1676DAE6C4ED7F4AB1CECBBB518032DC8552940EC000215000",
                   INITP_02 => X"FFFFFFFFD13122F102C4000020234E2A150A802842A2570818AA301601451725",
                   INITP_03 => X"002805A0067900001C01FF6DBA70003B2895002618D160468AA223451324AA15",
                   INITP_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_07 => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(  ADDRA => address_a(13 downto 0),
                   ENA => enable,
                  CLKA => clk,
                   DOA => data_out_a_ll(31 downto 0),
                  DOPA => data_out_a_ll(35 downto 32), 
                   DIA => data_in_a(31 downto 0),
                  DIPA => data_in_a(35 downto 32), 
                   WEA => "0000",
                REGCEA => '0',
                  RSTA => '0',
                 ADDRB => address_b(13 downto 0),
                   ENB => enable_b,
                  CLKB => clk_b,
                   DOB => data_out_b_ll(31 downto 0),
                  DOPB => data_out_b_ll(35 downto 32), 
                   DIB => data_in_b_ll(31 downto 0),
                  DIPB => data_in_b_ll(35 downto 32), 
                   WEB => we_b_l(3 downto 0),
                REGCEB => '0',
                  RSTB => '0');
      -- 
      kcpsm6_rom_lh: RAMB16BWER
      generic map ( DATA_WIDTH_A => 9,
                    DOA_REG => 0,
                    EN_RSTRAM_A => FALSE,
                    INIT_A => X"000000000",
                    RST_PRIORITY_A => "CE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    DATA_WIDTH_B => 9,
                    DOB_REG => 0,
                    EN_RSTRAM_B => FALSE,
                    INIT_B => X"000000000",
                    RST_PRIORITY_B => "CE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    RSTTYPE => "SYNC",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    SIM_DEVICE => "SPARTAN6",
                    INIT_00 => X"B0B801000101B068480B0C0C1090F9FCEC000400040000095858B8005B005800",
                    INIT_01 => X"091008080910080809288A28CAC88AF0CAC8CAC88A281AE8EAC8EA100090E8B8",
                    INIT_02 => X"1C10001C28010001000100095B2849B06949695B285B0028B0D9D8C800100808",
                    INIT_03 => X"09000900090009000900090009000900095B285B00002C0C000C280300030000",
                    INIT_04 => X"8B0051105B000900090009000900090009000900090009000900090009000900",
                    INIT_05 => X"0C5B005B5B0C005B0C0C5B000B0B0009000900095BB0E959A8E9000C0C28B0E3",
                    INIT_06 => X"0C005B005910000B0C28B088E85880EB5B0C000B0C000B0B0C0C005B000B0B00",
                    INIT_07 => X"91E991E9000C0C90E990E990E990E990E990E90C90E990E990E9000C0C10000B",
                    INIT_08 => X"000C000C000C000C000C0B000C000C0B2891E991E990E990E990E990E990E90C",
                    INIT_09 => X"BFBEBEBDBD000B0C0C10000B0C0C100100002C0B0B0CB8000B0C000C000C000C",
                    INIT_0A => X"090009000100015B000C0C050588FBEB90FBEBB9B0FFEEB8B7B6B8000C0C28BF",
                    INIT_0B => X"A0EFB8000B0C000C0C93835B0404B1B0F2E290E990E990FFEE9A8A8971000A0A",
                    INIT_0C => X"EE0FB0FAFAE95A5A28098FA0EF80EF3588FDEDB8B7B6B5B5B4B4B3B1B8A8B8EF",
                    INIT_0D => X"B80E10F08A8BB0E15290E9010C0C0B0AB0EEB0010D0DB4B4B3B0B0280D0D0EA8",
                    INIT_0E => X"0E06010B0C0C28B0E58D23B0C8A4A4A308C80000520D0D0B280FB8B4B4B3B5B5",
                    INIT_0F => X"E9280D0D0EB0E9280D0D0EB0E9100EB0E9D000280D0D0B0C0CB0E959B1010D0D",
                    INIT_10 => X"B1520F2891B8FFEE9F8EB800B1E9B8B1B8280D0D0ED000B0010D0D280D0D0EB0",
                    INIT_11 => X"B6B8B80101B811B80B0C0CB8250D0D01280F0E0111916F91E78FB89C9C8B01B8",
                    INIT_12 => X"0C0C91EE91EE0EB6B3B4B4010C010CB80A0A0E0B0C0CB6B5B5B8000C0C91EEB8",
                    INIT_13 => X"2C0CB801B8B1B1EC110000B1E80C00B1EC01B8100C010C01B80A0AB80C91EE00",
                    INIT_14 => X"0151BC0C7949794979000C79000C0C78B801B87C01097B7C1C91EB7C010A097C",
                    INIT_15 => X"CA0191E991EE9C9C8B01B828B89B8B5B00010001000100015BB8B411B8B1EC8C",
                    INIT_16 => X"0A0A119D8D8AA8E25388EA250A28B1FBEB000CA8E9000C000B0C000C28B8B1DA",
                    INIT_17 => X"0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A",
                    INIT_18 => X"1000B8B3B80C0C01100C0CE0E90B000C0CB00188E8B8B0B828B87FEE9F8EB80A",
                    INIT_19 => X"0CB8000C0C000B0C0CC8FBEB000C0C110C010CB80E0A0A0B0C0CB8000C0C000B",
                    INIT_1A => X"A86949D1FBEB000C0C91E991E9000C0C100B0C88E949110C010CB80E0A0A0B0C",
                    INIT_1B => X"83B8B3B3B2B2B85BB169496BB8B3B30B0BB80009000100015BB2B2000C0CB3B3",
                    INIT_1C => X"010A09B1F9E95959000109115B00290100090009000900095B100B0C000C0C93",
                    INIT_1D => X"09285BB1E1897100515B285B005B006959695969596959010A09010A09010A09",
                    INIT_1E => X"01000100090106285B118991E971000900010001000100091000010001000100",
                    INIT_1F => X"00001009100910000928B169010001000100010009012801B169010001000100",
                    INIT_20 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_21 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_22 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_23 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_24 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_25 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_26 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_27 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_28 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_29 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_30 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_31 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_32 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_33 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_34 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"A9554AA67E5224396232ACA65D55555555557CAB6AAD3F844455555D7E0C56DF",
                   INITP_01 => X"4515704C231070A0B250B0112095900EB2112A0C558489320462C12AAA94D554",
                   INITP_02 => X"FFFFFFFFE256552E6879AADC8AD9FB593AF13595281500657F199D231C3AC791",
                   INITP_03 => X"2B6D57B555D6AAD5737F552490DCAAC843986A90925492A064495032E99275C3",
                   INITP_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_07 => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(  ADDRA => address_a(13 downto 0),
                   ENA => enable,
                  CLKA => clk,
                   DOA => data_out_a_lh(31 downto 0),
                  DOPA => data_out_a_lh(35 downto 32), 
                   DIA => data_in_a(31 downto 0),
                  DIPA => data_in_a(35 downto 32), 
                   WEA => "0000",
                REGCEA => '0',
                  RSTA => '0',
                 ADDRB => address_b(13 downto 0),
                   ENB => enable_b,
                  CLKB => clk_b,
                   DOB => data_out_b_lh(31 downto 0),
                  DOPB => data_out_b_lh(35 downto 32), 
                   DIB => data_in_b_lh(31 downto 0),
                  DIPB => data_in_b_lh(35 downto 32), 
                   WEB => we_b_l(3 downto 0),
                REGCEB => '0',
                  RSTB => '0');
      --
      kcpsm6_rom_hl: RAMB16BWER
      generic map ( DATA_WIDTH_A => 9,
                    DOA_REG => 0,
                    EN_RSTRAM_A => FALSE,
                    INIT_A => X"000000000",
                    RST_PRIORITY_A => "CE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    DATA_WIDTH_B => 9,
                    DOB_REG => 0,
                    EN_RSTRAM_B => FALSE,
                    INIT_B => X"000000000",
                    RST_PRIORITY_B => "CE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    RSTTYPE => "SYNC",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    SIM_DEVICE => "SPARTAN6",
                    INIT_00 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_01 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_02 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_03 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_07 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_08 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_09 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_10 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_11 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_12 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_13 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_14 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_15 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_16 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_17 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_18 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_19 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_20 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_21 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_22 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_23 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_24 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_25 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_26 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_27 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_28 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_29 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_30 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_31 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_32 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_33 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_34 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_01 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_02 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_03 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_07 => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(  ADDRA => address_a(13 downto 0),
                   ENA => enable,
                  CLKA => clk,
                   DOA => data_out_a_hl(31 downto 0),
                  DOPA => data_out_a_hl(35 downto 32), 
                   DIA => data_in_a(31 downto 0),
                  DIPA => data_in_a(35 downto 32), 
                   WEA => "0000",
                REGCEA => '0',
                  RSTA => '0',
                 ADDRB => address_b(13 downto 0),
                   ENB => enable_b,
                  CLKB => clk_b,
                   DOB => data_out_b_hl(31 downto 0),
                  DOPB => data_out_b_hl(35 downto 32), 
                   DIB => data_in_b_hl(31 downto 0),
                  DIPB => data_in_b_hl(35 downto 32), 
                   WEB => we_b_h(3 downto 0),
                REGCEB => '0',
                  RSTB => '0');
      -- 
      kcpsm6_rom_hh: RAMB16BWER
      generic map ( DATA_WIDTH_A => 9,
                    DOA_REG => 0,
                    EN_RSTRAM_A => FALSE,
                    INIT_A => X"000000000",
                    RST_PRIORITY_A => "CE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    DATA_WIDTH_B => 9,
                    DOB_REG => 0,
                    EN_RSTRAM_B => FALSE,
                    INIT_B => X"000000000",
                    RST_PRIORITY_B => "CE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    RSTTYPE => "SYNC",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    SIM_DEVICE => "SPARTAN6",
                    INIT_00 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_01 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_02 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_03 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_07 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_08 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_09 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_10 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_11 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_12 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_13 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_14 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_15 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_16 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_17 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_18 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_19 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_20 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_21 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_22 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_23 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_24 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_25 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_26 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_27 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_28 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_29 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_30 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_31 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_32 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_33 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_34 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_01 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_02 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_03 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_07 => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(  ADDRA => address_a(13 downto 0),
                   ENA => enable,
                  CLKA => clk,
                   DOA => data_out_a_hh(31 downto 0),
                  DOPA => data_out_a_hh(35 downto 32), 
                   DIA => data_in_a(31 downto 0),
                  DIPA => data_in_a(35 downto 32), 
                   WEA => "0000",
                REGCEA => '0',
                  RSTA => '0',
                 ADDRB => address_b(13 downto 0),
                   ENB => enable_b,
                  CLKB => clk_b,
                   DOB => data_out_b_hh(31 downto 0),
                  DOPB => data_out_b_hh(35 downto 32), 
                   DIB => data_in_b_hh(31 downto 0),
                  DIPB => data_in_b_hh(35 downto 32), 
                   WEB => we_b_h(3 downto 0),
                REGCEB => '0',
                  RSTB => '0');
    --
    end generate s6;
    --
    --
    v6 : if (C_FAMILY = "V6") generate
      --
      address_a <= '1' & address(11 downto 0) & "111";
      instruction <= data_out_a_h(32) & data_out_a_h(7 downto 0) & data_out_a_l(32) & data_out_a_l(7 downto 0);
      data_in_a <= "000000000000000000000000000000000000";
      jtag_dout <= data_out_b_h(32) & data_out_b_h(7 downto 0) & data_out_b_l(32) & data_out_b_l(7 downto 0);
      --
      no_loader : if (C_JTAG_LOADER_ENABLE = 0) generate
        data_in_b_l <= "000" & data_out_b_l(32) & "000000000000000000000000" & data_out_b_l(7 downto 0);
        data_in_b_h <= "000" & data_out_b_h(32) & "000000000000000000000000" & data_out_b_h(7 downto 0);
        address_b <= "1111111111111111";
        we_b <= "00000000";
        enable_b <= '0';
        rdl <= '0';
        clk_b <= '0';
      end generate no_loader;
      --
      loader : if (C_JTAG_LOADER_ENABLE = 1) generate
        data_in_b_h <= "000" & jtag_din(17) & "000000000000000000000000" & jtag_din(16 downto 9);
        data_in_b_l <= "000" & jtag_din(8) & "000000000000000000000000" & jtag_din(7 downto 0);
        address_b <= '1' & jtag_addr(11 downto 0) & "111";
        we_b <= jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we;
        enable_b <= jtag_en(0);
        rdl <= rdl_bus(0);
        clk_b <= jtag_clk;
      end generate loader;
      --
      kcpsm6_rom_l: RAMB36E1
      generic map ( READ_WIDTH_A => 9,
                    WRITE_WIDTH_A => 9,
                    DOA_REG => 0,
                    INIT_A => X"000000000",
                    RSTREG_PRIORITY_A => "REGCE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    READ_WIDTH_B => 9,
                    WRITE_WIDTH_B => 9,
                    DOB_REG => 0,
                    INIT_B => X"000000000",
                    RSTREG_PRIORITY_B => "REGCE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    RAM_MODE => "TDP",
                    RDADDR_COLLISION_HWCONFIG => "DELAYED_WRITE",
                    EN_ECC_READ => FALSE,
                    EN_ECC_WRITE => FALSE,
                    RAM_EXTENSION_A => "NONE",
                    RAM_EXTENSION_B => "NONE",
                    SIM_DEVICE => "VIRTEX6",
                    INIT_00 => X"100195A3C8EA1B4000000007131415202049204C204C4C9F0131FF3F743B743F",
                    INIT_01 => X"9843400D0343102700000A00F60007351100E900B900DF007B006123E313FF00",
                    INIT_02 => X"FB6B53FB00704D804D904D005400044E0105086400744C004300000100438096",
                    INIT_03 => X"FF4D014D014DA84DC04D804D044D004D005400744D53041F691E00206B204D53",
                    INIT_04 => X"014D6010744D024D014DA84DC04D034D024D014DDC4D084D004D004DFF4DFF4D",
                    INIT_05 => X"0C542D15140469130800749D13004D044D004D00546E800000045C0039009D70",
                    INIT_06 => X"013F743B7469371001001000FF1C6901261D69041B2DE803190010749D1C1653",
                    INIT_07 => X"080050175C08033713D71DD71BD71AD718DF1C2C42173200F9145C2803693708",
                    INIT_08 => X"66C866A86688666866480066286608080001151813D71DD71BD71AD718DB1C0C",
                    INIT_09 => X"000000000069022C013769012C016B604D530450000400690100D728D70866E8",
                    INIT_0A => X"004D384D604D70545F282870600000008300000050FF0400F0C0015F28260000",
                    INIT_0B => X"91FF016940012D2C2890807450403062B0A071FF710A6CFF04000101304D0000",
                    INIT_0C => X"80FEA50A0D0201000000019800DA00A000000000F0C0B0A090807030010000FF",
                    INIT_0D => X"0000A1B40101BF5040BE00BF07004015BE80BFD3EC02908070B0A00034020000",
                    INIT_0E => X"0020BF00000700CBB00150D00100000604002B25A00D07000000019080701000",
                    INIT_0F => X"72003E0205FF73003A0203F965EA02F362A1C8003E02000006ED200505D3DE02",
                    INIT_10 => X"40E000003E001FFF000101371D00003001000F0204A1C8A1D3E40200300203EA",
                    INIT_11 => X"C00100FCEA013E0000FE0601A031023E000000FC1E1E0F2C300100000001D901",
                    INIT_12 => X"28246B036B0400C0D0B0A0C42CB53400401F0000FF06708090015F28245E0000",
                    INIT_13 => X"8000018700807F027ED7DF7E000C8078019001DF2CC42CB500401F01346D055F",
                    INIT_14 => X"A58034000D060C070B5C030A5C08000901B10000B60002007F900000B6090000",
                    INIT_15 => X"01A5C200BE02000001BF0100010001744D804D904D604D7054002069009E0E01",
                    INIT_16 => X"4547D40001010060400000A00000C920005F2000175C036920012D240000B600",
                    INIT_17 => X"6369736142203A6E6F6974617A69726F68747541005F2F2054534F50002F2054",
                    INIT_18 => X"37690010010C0101690C016901015C001D37010000001001000003FF00010100",
                    INIT_19 => X"07015F08246903001D00005B5F0826C40CB51400024000800007015F08246902",
                    INIT_1A => X"0002054A18005F0826310220015C001D692201000003C40CB51400021E00C000",
                    INIT_1B => X"4000706050400174750705100070601800014D184D604D705460705F08287060",
                    INIT_1C => X"B62C27B10E801D003B8700B1744D02304D004D604D004D00346940012D0C2850",
                    INIT_1D => X"030074B74001304D303400743BF43BE036C037A0388039B6261DB63631B6312C",
                    INIT_1E => X"804C904C02FA200074D101D7FF304C004C704C804C904C03494C704C804C904C",
                    INIT_1F => X"000049044906494C0500F301F749704C804C904CD8FA00FCE501F749D04C704C",
                    INIT_20 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_21 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_22 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_23 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_24 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_25 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_26 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_27 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_28 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_29 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_30 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_31 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_32 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_33 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_34 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_40 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_41 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_42 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_43 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_44 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_45 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_46 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_47 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_48 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_49 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_50 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_51 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_52 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_53 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_54 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_55 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_56 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_57 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_58 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_59 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_60 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_61 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_62 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_63 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_64 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_65 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_66 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_67 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_68 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_69 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_70 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_71 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_72 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_73 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_74 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_75 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_76 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_77 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_78 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_79 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"A3000A0981ADDDA4AD480008100000000000014400000021112AAAA2AC508822",
                   INITP_01 => X"145566C175350D1676DAE6C4ED7F4AB1CECBBB518032DC8552940EC000215000",
                   INITP_02 => X"FFFFFFFFD13122F102C4000020234E2A150A802842A2570818AA301601451725",
                   INITP_03 => X"002805A0067900001C01FF6DBA70003B2895002618D160468AA223451324AA15",
                   INITP_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_07 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_08 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_09 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0A => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0B => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0C => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0D => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0E => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0F => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(   ADDRARDADDR => address_a,
                      ENARDEN => enable,
                    CLKARDCLK => clk,
                        DOADO => data_out_a_l(31 downto 0),
                      DOPADOP => data_out_a_l(35 downto 32), 
                        DIADI => data_in_a(31 downto 0),
                      DIPADIP => data_in_a(35 downto 32), 
                          WEA => "0000",
                  REGCEAREGCE => '0',
                RSTRAMARSTRAM => '0',
                RSTREGARSTREG => '0',
                  ADDRBWRADDR => address_b,
                      ENBWREN => enable_b,
                    CLKBWRCLK => clk_b,
                        DOBDO => data_out_b_l(31 downto 0),
                      DOPBDOP => data_out_b_l(35 downto 32), 
                        DIBDI => data_in_b_l(31 downto 0),
                      DIPBDIP => data_in_b_l(35 downto 32), 
                        WEBWE => we_b,
                       REGCEB => '0',
                      RSTRAMB => '0',
                      RSTREGB => '0',
                   CASCADEINA => '0',
                   CASCADEINB => '0',
                INJECTDBITERR => '0',
                INJECTSBITERR => '0');
      --
      kcpsm6_rom_h: RAMB36E1
      generic map ( READ_WIDTH_A => 9,
                    WRITE_WIDTH_A => 9,
                    DOA_REG => 0,
                    INIT_A => X"000000000",
                    RSTREG_PRIORITY_A => "REGCE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    READ_WIDTH_B => 9,
                    WRITE_WIDTH_B => 9,
                    DOB_REG => 0,
                    INIT_B => X"000000000",
                    RSTREG_PRIORITY_B => "REGCE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    RAM_MODE => "TDP",
                    RDADDR_COLLISION_HWCONFIG => "DELAYED_WRITE",
                    EN_ECC_READ => FALSE,
                    EN_ECC_WRITE => FALSE,
                    RAM_EXTENSION_A => "NONE",
                    RAM_EXTENSION_B => "NONE",
                    SIM_DEVICE => "VIRTEX6",
                    INIT_00 => X"B0B801000101B068480B0C0C1090F9FCEC000400040000095858B8005B005800",
                    INIT_01 => X"091008080910080809288A28CAC88AF0CAC8CAC88A281AE8EAC8EA100090E8B8",
                    INIT_02 => X"1C10001C28010001000100095B2849B06949695B285B0028B0D9D8C800100808",
                    INIT_03 => X"09000900090009000900090009000900095B285B00002C0C000C280300030000",
                    INIT_04 => X"8B0051105B000900090009000900090009000900090009000900090009000900",
                    INIT_05 => X"0C5B005B5B0C005B0C0C5B000B0B0009000900095BB0E959A8E9000C0C28B0E3",
                    INIT_06 => X"0C005B005910000B0C28B088E85880EB5B0C000B0C000B0B0C0C005B000B0B00",
                    INIT_07 => X"91E991E9000C0C90E990E990E990E990E990E90C90E990E990E9000C0C10000B",
                    INIT_08 => X"000C000C000C000C000C0B000C000C0B2891E991E990E990E990E990E990E90C",
                    INIT_09 => X"BFBEBEBDBD000B0C0C10000B0C0C100100002C0B0B0CB8000B0C000C000C000C",
                    INIT_0A => X"090009000100015B000C0C050588FBEB90FBEBB9B0FFEEB8B7B6B8000C0C28BF",
                    INIT_0B => X"A0EFB8000B0C000C0C93835B0404B1B0F2E290E990E990FFEE9A8A8971000A0A",
                    INIT_0C => X"EE0FB0FAFAE95A5A28098FA0EF80EF3588FDEDB8B7B6B5B5B4B4B3B1B8A8B8EF",
                    INIT_0D => X"B80E10F08A8BB0E15290E9010C0C0B0AB0EEB0010D0DB4B4B3B0B0280D0D0EA8",
                    INIT_0E => X"0E06010B0C0C28B0E58D23B0C8A4A4A308C80000520D0D0B280FB8B4B4B3B5B5",
                    INIT_0F => X"E9280D0D0EB0E9280D0D0EB0E9100EB0E9D000280D0D0B0C0CB0E959B1010D0D",
                    INIT_10 => X"B1520F2891B8FFEE9F8EB800B1E9B8B1B8280D0D0ED000B0010D0D280D0D0EB0",
                    INIT_11 => X"B6B8B80101B811B80B0C0CB8250D0D01280F0E0111916F91E78FB89C9C8B01B8",
                    INIT_12 => X"0C0C91EE91EE0EB6B3B4B4010C010CB80A0A0E0B0C0CB6B5B5B8000C0C91EEB8",
                    INIT_13 => X"2C0CB801B8B1B1EC110000B1E80C00B1EC01B8100C010C01B80A0AB80C91EE00",
                    INIT_14 => X"0151BC0C7949794979000C79000C0C78B801B87C01097B7C1C91EB7C010A097C",
                    INIT_15 => X"CA0191E991EE9C9C8B01B828B89B8B5B00010001000100015BB8B411B8B1EC8C",
                    INIT_16 => X"0A0A119D8D8AA8E25388EA250A28B1FBEB000CA8E9000C000B0C000C28B8B1DA",
                    INIT_17 => X"0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A",
                    INIT_18 => X"1000B8B3B80C0C01100C0CE0E90B000C0CB00188E8B8B0B828B87FEE9F8EB80A",
                    INIT_19 => X"0CB8000C0C000B0C0CC8FBEB000C0C110C010CB80E0A0A0B0C0CB8000C0C000B",
                    INIT_1A => X"A86949D1FBEB000C0C91E991E9000C0C100B0C88E949110C010CB80E0A0A0B0C",
                    INIT_1B => X"83B8B3B3B2B2B85BB169496BB8B3B30B0BB80009000100015BB2B2000C0CB3B3",
                    INIT_1C => X"010A09B1F9E95959000109115B00290100090009000900095B100B0C000C0C93",
                    INIT_1D => X"09285BB1E1897100515B285B005B006959695969596959010A09010A09010A09",
                    INIT_1E => X"01000100090106285B118991E971000900010001000100091000010001000100",
                    INIT_1F => X"00001009100910000928B169010001000100010009012801B169010001000100",
                    INIT_20 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_21 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_22 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_23 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_24 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_25 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_26 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_27 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_28 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_29 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_30 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_31 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_32 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_33 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_34 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_40 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_41 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_42 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_43 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_44 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_45 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_46 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_47 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_48 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_49 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_50 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_51 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_52 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_53 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_54 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_55 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_56 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_57 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_58 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_59 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_60 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_61 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_62 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_63 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_64 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_65 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_66 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_67 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_68 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_69 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_70 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_71 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_72 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_73 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_74 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_75 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_76 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_77 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_78 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_79 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"A9554AA67E5224396232ACA65D55555555557CAB6AAD3F844455555D7E0C56DF",
                   INITP_01 => X"4515704C231070A0B250B0112095900EB2112A0C558489320462C12AAA94D554",
                   INITP_02 => X"FFFFFFFFE256552E6879AADC8AD9FB593AF13595281500657F199D231C3AC791",
                   INITP_03 => X"2B6D57B555D6AAD5737F552490DCAAC843986A90925492A064495032E99275C3",
                   INITP_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_07 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_08 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_09 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0A => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0B => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0C => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0D => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0E => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0F => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(   ADDRARDADDR => address_a,
                      ENARDEN => enable,
                    CLKARDCLK => clk,
                        DOADO => data_out_a_h(31 downto 0),
                      DOPADOP => data_out_a_h(35 downto 32), 
                        DIADI => data_in_a(31 downto 0),
                      DIPADIP => data_in_a(35 downto 32), 
                          WEA => "0000",
                  REGCEAREGCE => '0',
                RSTRAMARSTRAM => '0',
                RSTREGARSTREG => '0',
                  ADDRBWRADDR => address_b,
                      ENBWREN => enable_b,
                    CLKBWRCLK => clk_b,
                        DOBDO => data_out_b_h(31 downto 0),
                      DOPBDOP => data_out_b_h(35 downto 32), 
                        DIBDI => data_in_b_h(31 downto 0),
                      DIPBDIP => data_in_b_h(35 downto 32), 
                        WEBWE => we_b,
                       REGCEB => '0',
                      RSTRAMB => '0',
                      RSTREGB => '0',
                   CASCADEINA => '0',
                   CASCADEINB => '0',
                INJECTDBITERR => '0',
                INJECTSBITERR => '0');
      --
    end generate v6;
    --
    --
    akv7 : if (C_FAMILY = "7S") generate
      --
      address_a <= '1' & address(11 downto 0) & "111";
      instruction <= data_out_a_h(32) & data_out_a_h(7 downto 0) & data_out_a_l(32) & data_out_a_l(7 downto 0);
      data_in_a <= "000000000000000000000000000000000000";
      jtag_dout <= data_out_b_h(32) & data_out_b_h(7 downto 0) & data_out_b_l(32) & data_out_b_l(7 downto 0);
      --
      no_loader : if (C_JTAG_LOADER_ENABLE = 0) generate
        data_in_b_l <= "000" & data_out_b_l(32) & "000000000000000000000000" & data_out_b_l(7 downto 0);
        data_in_b_h <= "000" & data_out_b_h(32) & "000000000000000000000000" & data_out_b_h(7 downto 0);
        address_b <= "1111111111111111";
        we_b <= "00000000";
        enable_b <= '0';
        rdl <= '0';
        clk_b <= '0';
      end generate no_loader;
      --
      loader : if (C_JTAG_LOADER_ENABLE = 1) generate
        data_in_b_h <= "000" & jtag_din(17) & "000000000000000000000000" & jtag_din(16 downto 9);
        data_in_b_l <= "000" & jtag_din(8) & "000000000000000000000000" & jtag_din(7 downto 0);
        address_b <= '1' & jtag_addr(11 downto 0) & "111";
        we_b <= jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we;
        enable_b <= jtag_en(0);
        rdl <= rdl_bus(0);
        clk_b <= jtag_clk;
      end generate loader;
      --
      kcpsm6_rom_l: RAMB36E1
      generic map ( READ_WIDTH_A => 9,
                    WRITE_WIDTH_A => 9,
                    DOA_REG => 0,
                    INIT_A => X"000000000",
                    RSTREG_PRIORITY_A => "REGCE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    READ_WIDTH_B => 9,
                    WRITE_WIDTH_B => 9,
                    DOB_REG => 0,
                    INIT_B => X"000000000",
                    RSTREG_PRIORITY_B => "REGCE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    RAM_MODE => "TDP",
                    RDADDR_COLLISION_HWCONFIG => "DELAYED_WRITE",
                    EN_ECC_READ => FALSE,
                    EN_ECC_WRITE => FALSE,
                    RAM_EXTENSION_A => "NONE",
                    RAM_EXTENSION_B => "NONE",
                    SIM_DEVICE => "7SERIES",
                    INIT_00 => X"100195A3C8EA1B4000000007131415202049204C204C4C9F0131FF3F743B743F",
                    INIT_01 => X"9843400D0343102700000A00F60007351100E900B900DF007B006123E313FF00",
                    INIT_02 => X"FB6B53FB00704D804D904D005400044E0105086400744C004300000100438096",
                    INIT_03 => X"FF4D014D014DA84DC04D804D044D004D005400744D53041F691E00206B204D53",
                    INIT_04 => X"014D6010744D024D014DA84DC04D034D024D014DDC4D084D004D004DFF4DFF4D",
                    INIT_05 => X"0C542D15140469130800749D13004D044D004D00546E800000045C0039009D70",
                    INIT_06 => X"013F743B7469371001001000FF1C6901261D69041B2DE803190010749D1C1653",
                    INIT_07 => X"080050175C08033713D71DD71BD71AD718DF1C2C42173200F9145C2803693708",
                    INIT_08 => X"66C866A86688666866480066286608080001151813D71DD71BD71AD718DB1C0C",
                    INIT_09 => X"000000000069022C013769012C016B604D530450000400690100D728D70866E8",
                    INIT_0A => X"004D384D604D70545F282870600000008300000050FF0400F0C0015F28260000",
                    INIT_0B => X"91FF016940012D2C2890807450403062B0A071FF710A6CFF04000101304D0000",
                    INIT_0C => X"80FEA50A0D0201000000019800DA00A000000000F0C0B0A090807030010000FF",
                    INIT_0D => X"0000A1B40101BF5040BE00BF07004015BE80BFD3EC02908070B0A00034020000",
                    INIT_0E => X"0020BF00000700CBB00150D00100000604002B25A00D07000000019080701000",
                    INIT_0F => X"72003E0205FF73003A0203F965EA02F362A1C8003E02000006ED200505D3DE02",
                    INIT_10 => X"40E000003E001FFF000101371D00003001000F0204A1C8A1D3E40200300203EA",
                    INIT_11 => X"C00100FCEA013E0000FE0601A031023E000000FC1E1E0F2C300100000001D901",
                    INIT_12 => X"28246B036B0400C0D0B0A0C42CB53400401F0000FF06708090015F28245E0000",
                    INIT_13 => X"8000018700807F027ED7DF7E000C8078019001DF2CC42CB500401F01346D055F",
                    INIT_14 => X"A58034000D060C070B5C030A5C08000901B10000B60002007F900000B6090000",
                    INIT_15 => X"01A5C200BE02000001BF0100010001744D804D904D604D7054002069009E0E01",
                    INIT_16 => X"4547D40001010060400000A00000C920005F2000175C036920012D240000B600",
                    INIT_17 => X"6369736142203A6E6F6974617A69726F68747541005F2F2054534F50002F2054",
                    INIT_18 => X"37690010010C0101690C016901015C001D37010000001001000003FF00010100",
                    INIT_19 => X"07015F08246903001D00005B5F0826C40CB51400024000800007015F08246902",
                    INIT_1A => X"0002054A18005F0826310220015C001D692201000003C40CB51400021E00C000",
                    INIT_1B => X"4000706050400174750705100070601800014D184D604D705460705F08287060",
                    INIT_1C => X"B62C27B10E801D003B8700B1744D02304D004D604D004D00346940012D0C2850",
                    INIT_1D => X"030074B74001304D303400743BF43BE036C037A0388039B6261DB63631B6312C",
                    INIT_1E => X"804C904C02FA200074D101D7FF304C004C704C804C904C03494C704C804C904C",
                    INIT_1F => X"000049044906494C0500F301F749704C804C904CD8FA00FCE501F749D04C704C",
                    INIT_20 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_21 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_22 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_23 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_24 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_25 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_26 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_27 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_28 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_29 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_30 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_31 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_32 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_33 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_34 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_40 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_41 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_42 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_43 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_44 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_45 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_46 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_47 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_48 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_49 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_50 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_51 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_52 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_53 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_54 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_55 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_56 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_57 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_58 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_59 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_60 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_61 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_62 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_63 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_64 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_65 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_66 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_67 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_68 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_69 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_70 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_71 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_72 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_73 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_74 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_75 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_76 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_77 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_78 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_79 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"A3000A0981ADDDA4AD480008100000000000014400000021112AAAA2AC508822",
                   INITP_01 => X"145566C175350D1676DAE6C4ED7F4AB1CECBBB518032DC8552940EC000215000",
                   INITP_02 => X"FFFFFFFFD13122F102C4000020234E2A150A802842A2570818AA301601451725",
                   INITP_03 => X"002805A0067900001C01FF6DBA70003B2895002618D160468AA223451324AA15",
                   INITP_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_07 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_08 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_09 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0A => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0B => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0C => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0D => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0E => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0F => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(   ADDRARDADDR => address_a,
                      ENARDEN => enable,
                    CLKARDCLK => clk,
                        DOADO => data_out_a_l(31 downto 0),
                      DOPADOP => data_out_a_l(35 downto 32), 
                        DIADI => data_in_a(31 downto 0),
                      DIPADIP => data_in_a(35 downto 32), 
                          WEA => "0000",
                  REGCEAREGCE => '0',
                RSTRAMARSTRAM => '0',
                RSTREGARSTREG => '0',
                  ADDRBWRADDR => address_b,
                      ENBWREN => enable_b,
                    CLKBWRCLK => clk_b,
                        DOBDO => data_out_b_l(31 downto 0),
                      DOPBDOP => data_out_b_l(35 downto 32), 
                        DIBDI => data_in_b_l(31 downto 0),
                      DIPBDIP => data_in_b_l(35 downto 32), 
                        WEBWE => we_b,
                       REGCEB => '0',
                      RSTRAMB => '0',
                      RSTREGB => '0',
                   CASCADEINA => '0',
                   CASCADEINB => '0',
                INJECTDBITERR => '0',
                INJECTSBITERR => '0');
      --
      kcpsm6_rom_h: RAMB36E1
      generic map ( READ_WIDTH_A => 9,
                    WRITE_WIDTH_A => 9,
                    DOA_REG => 0,
                    INIT_A => X"000000000",
                    RSTREG_PRIORITY_A => "REGCE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    READ_WIDTH_B => 9,
                    WRITE_WIDTH_B => 9,
                    DOB_REG => 0,
                    INIT_B => X"000000000",
                    RSTREG_PRIORITY_B => "REGCE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    RAM_MODE => "TDP",
                    RDADDR_COLLISION_HWCONFIG => "DELAYED_WRITE",
                    EN_ECC_READ => FALSE,
                    EN_ECC_WRITE => FALSE,
                    RAM_EXTENSION_A => "NONE",
                    RAM_EXTENSION_B => "NONE",
                    SIM_DEVICE => "7SERIES",
                    INIT_00 => X"B0B801000101B068480B0C0C1090F9FCEC000400040000095858B8005B005800",
                    INIT_01 => X"091008080910080809288A28CAC88AF0CAC8CAC88A281AE8EAC8EA100090E8B8",
                    INIT_02 => X"1C10001C28010001000100095B2849B06949695B285B0028B0D9D8C800100808",
                    INIT_03 => X"09000900090009000900090009000900095B285B00002C0C000C280300030000",
                    INIT_04 => X"8B0051105B000900090009000900090009000900090009000900090009000900",
                    INIT_05 => X"0C5B005B5B0C005B0C0C5B000B0B0009000900095BB0E959A8E9000C0C28B0E3",
                    INIT_06 => X"0C005B005910000B0C28B088E85880EB5B0C000B0C000B0B0C0C005B000B0B00",
                    INIT_07 => X"91E991E9000C0C90E990E990E990E990E990E90C90E990E990E9000C0C10000B",
                    INIT_08 => X"000C000C000C000C000C0B000C000C0B2891E991E990E990E990E990E990E90C",
                    INIT_09 => X"BFBEBEBDBD000B0C0C10000B0C0C100100002C0B0B0CB8000B0C000C000C000C",
                    INIT_0A => X"090009000100015B000C0C050588FBEB90FBEBB9B0FFEEB8B7B6B8000C0C28BF",
                    INIT_0B => X"A0EFB8000B0C000C0C93835B0404B1B0F2E290E990E990FFEE9A8A8971000A0A",
                    INIT_0C => X"EE0FB0FAFAE95A5A28098FA0EF80EF3588FDEDB8B7B6B5B5B4B4B3B1B8A8B8EF",
                    INIT_0D => X"B80E10F08A8BB0E15290E9010C0C0B0AB0EEB0010D0DB4B4B3B0B0280D0D0EA8",
                    INIT_0E => X"0E06010B0C0C28B0E58D23B0C8A4A4A308C80000520D0D0B280FB8B4B4B3B5B5",
                    INIT_0F => X"E9280D0D0EB0E9280D0D0EB0E9100EB0E9D000280D0D0B0C0CB0E959B1010D0D",
                    INIT_10 => X"B1520F2891B8FFEE9F8EB800B1E9B8B1B8280D0D0ED000B0010D0D280D0D0EB0",
                    INIT_11 => X"B6B8B80101B811B80B0C0CB8250D0D01280F0E0111916F91E78FB89C9C8B01B8",
                    INIT_12 => X"0C0C91EE91EE0EB6B3B4B4010C010CB80A0A0E0B0C0CB6B5B5B8000C0C91EEB8",
                    INIT_13 => X"2C0CB801B8B1B1EC110000B1E80C00B1EC01B8100C010C01B80A0AB80C91EE00",
                    INIT_14 => X"0151BC0C7949794979000C79000C0C78B801B87C01097B7C1C91EB7C010A097C",
                    INIT_15 => X"CA0191E991EE9C9C8B01B828B89B8B5B00010001000100015BB8B411B8B1EC8C",
                    INIT_16 => X"0A0A119D8D8AA8E25388EA250A28B1FBEB000CA8E9000C000B0C000C28B8B1DA",
                    INIT_17 => X"0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A",
                    INIT_18 => X"1000B8B3B80C0C01100C0CE0E90B000C0CB00188E8B8B0B828B87FEE9F8EB80A",
                    INIT_19 => X"0CB8000C0C000B0C0CC8FBEB000C0C110C010CB80E0A0A0B0C0CB8000C0C000B",
                    INIT_1A => X"A86949D1FBEB000C0C91E991E9000C0C100B0C88E949110C010CB80E0A0A0B0C",
                    INIT_1B => X"83B8B3B3B2B2B85BB169496BB8B3B30B0BB80009000100015BB2B2000C0CB3B3",
                    INIT_1C => X"010A09B1F9E95959000109115B00290100090009000900095B100B0C000C0C93",
                    INIT_1D => X"09285BB1E1897100515B285B005B006959695969596959010A09010A09010A09",
                    INIT_1E => X"01000100090106285B118991E971000900010001000100091000010001000100",
                    INIT_1F => X"00001009100910000928B169010001000100010009012801B169010001000100",
                    INIT_20 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_21 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_22 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_23 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_24 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_25 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_26 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_27 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_28 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_29 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_30 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_31 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_32 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_33 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_34 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_40 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_41 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_42 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_43 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_44 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_45 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_46 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_47 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_48 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_49 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_50 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_51 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_52 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_53 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_54 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_55 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_56 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_57 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_58 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_59 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_60 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_61 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_62 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_63 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_64 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_65 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_66 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_67 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_68 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_69 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_70 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_71 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_72 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_73 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_74 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_75 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_76 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_77 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_78 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_79 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"A9554AA67E5224396232ACA65D55555555557CAB6AAD3F844455555D7E0C56DF",
                   INITP_01 => X"4515704C231070A0B250B0112095900EB2112A0C558489320462C12AAA94D554",
                   INITP_02 => X"FFFFFFFFE256552E6879AADC8AD9FB593AF13595281500657F199D231C3AC791",
                   INITP_03 => X"2B6D57B555D6AAD5737F552490DCAAC843986A90925492A064495032E99275C3",
                   INITP_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_07 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_08 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_09 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0A => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0B => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0C => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0D => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0E => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0F => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(   ADDRARDADDR => address_a,
                      ENARDEN => enable,
                    CLKARDCLK => clk,
                        DOADO => data_out_a_h(31 downto 0),
                      DOPADOP => data_out_a_h(35 downto 32), 
                        DIADI => data_in_a(31 downto 0),
                      DIPADIP => data_in_a(35 downto 32), 
                          WEA => "0000",
                  REGCEAREGCE => '0',
                RSTRAMARSTRAM => '0',
                RSTREGARSTREG => '0',
                  ADDRBWRADDR => address_b,
                      ENBWREN => enable_b,
                    CLKBWRCLK => clk_b,
                        DOBDO => data_out_b_h(31 downto 0),
                      DOPBDOP => data_out_b_h(35 downto 32), 
                        DIBDI => data_in_b_h(31 downto 0),
                      DIPBDIP => data_in_b_h(35 downto 32), 
                        WEBWE => we_b,
                       REGCEB => '0',
                      RSTRAMB => '0',
                      RSTREGB => '0',
                   CASCADEINA => '0',
                   CASCADEINB => '0',
                INJECTDBITERR => '0',
                INJECTSBITERR => '0');
      --
    end generate akv7;
    --
  end generate ram_4k_generate;	              
  --
  --
  --
  --
  -- JTAG Loader
  --
  instantiate_loader : if (C_JTAG_LOADER_ENABLE = 1) generate
  --
    jtag_loader_6_inst : jtag_loader_6
    generic map(              C_FAMILY => C_FAMILY,
                       C_NUM_PICOBLAZE => 1,
                  C_JTAG_LOADER_ENABLE => C_JTAG_LOADER_ENABLE,
                 C_BRAM_MAX_ADDR_WIDTH => BRAM_ADDRESS_WIDTH,
	                  C_ADDR_WIDTH_0 => BRAM_ADDRESS_WIDTH)
    port map( picoblaze_reset => rdl_bus,
                      jtag_en => jtag_en,
                     jtag_din => jtag_din,
                    jtag_addr => jtag_addr(BRAM_ADDRESS_WIDTH-1 downto 0),
                     jtag_clk => jtag_clk,
                      jtag_we => jtag_we,
                  jtag_dout_0 => jtag_dout,
                  jtag_dout_1 => jtag_dout, -- ports 1-7 are not used
                  jtag_dout_2 => jtag_dout, -- in a 1 device debug 
                  jtag_dout_3 => jtag_dout, -- session.  However, Synplify
                  jtag_dout_4 => jtag_dout, -- etc require all ports to
                  jtag_dout_5 => jtag_dout, -- be connected
                  jtag_dout_6 => jtag_dout,
                  jtag_dout_7 => jtag_dout);
    --  
  end generate instantiate_loader;
  --
end low_level_definition;
--
--
-------------------------------------------------------------------------------------------
--
-- JTAG Loader 
--
-------------------------------------------------------------------------------------------
--
--
-- JTAG Loader 6 - Version 6.00
-- Kris Chaplin 4 February 2010
-- Ken Chapman 15 August 2011 - Revised coding style
--
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
--
library unisim;
use unisim.vcomponents.all;
--
entity jtag_loader_6 is
generic(              C_JTAG_LOADER_ENABLE : integer := 1;
                                  C_FAMILY : string := "V6";
                           C_NUM_PICOBLAZE : integer := 1;
                     C_BRAM_MAX_ADDR_WIDTH : integer := 10;
        C_PICOBLAZE_INSTRUCTION_DATA_WIDTH : integer := 18;
                              C_JTAG_CHAIN : integer := 2;
                            C_ADDR_WIDTH_0 : integer := 10;
                            C_ADDR_WIDTH_1 : integer := 10;
                            C_ADDR_WIDTH_2 : integer := 10;
                            C_ADDR_WIDTH_3 : integer := 10;
                            C_ADDR_WIDTH_4 : integer := 10;
                            C_ADDR_WIDTH_5 : integer := 10;
                            C_ADDR_WIDTH_6 : integer := 10;
                            C_ADDR_WIDTH_7 : integer := 10);
port(   picoblaze_reset : out std_logic_vector(C_NUM_PICOBLAZE-1 downto 0);
                jtag_en : out std_logic_vector(C_NUM_PICOBLAZE-1 downto 0) := (others => '0');
               jtag_din : out std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0) := (others => '0');
              jtag_addr : out std_logic_vector(C_BRAM_MAX_ADDR_WIDTH-1 downto 0) := (others => '0');
               jtag_clk : out std_logic := '0';
                jtag_we : out std_logic := '0';
            jtag_dout_0 : in  std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
            jtag_dout_1 : in  std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
            jtag_dout_2 : in  std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
            jtag_dout_3 : in  std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
            jtag_dout_4 : in  std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
            jtag_dout_5 : in  std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
            jtag_dout_6 : in  std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
            jtag_dout_7 : in  std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0));
end jtag_loader_6;
--
architecture Behavioral of jtag_loader_6 is
  --
  signal num_picoblaze       : std_logic_vector(2 downto 0);
  signal picoblaze_instruction_data_width : std_logic_vector(4 downto 0);
  --
  signal drck                : std_logic;
  signal shift_clk           : std_logic;
  signal shift_din           : std_logic;
  signal shift_dout          : std_logic;
  signal shift               : std_logic;
  signal capture             : std_logic;
  --
  signal control_reg_ce      : std_logic;
  signal bram_ce             : std_logic_vector(C_NUM_PICOBLAZE-1 downto 0);
  signal bus_zero            : std_logic_vector(C_NUM_PICOBLAZE-1 downto 0) := (others => '0');
  signal jtag_en_int         : std_logic_vector(C_NUM_PICOBLAZE-1 downto 0);
  signal jtag_en_expanded    : std_logic_vector(7 downto 0) := (others => '0');
  signal jtag_addr_int       : std_logic_vector(C_BRAM_MAX_ADDR_WIDTH-1 downto 0);
  signal jtag_din_int        : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
  signal control_din         : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0):= (others => '0');
  signal control_dout        : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0):= (others => '0');
  signal control_dout_int    : std_logic_vector(7 downto 0):= (others => '0');
  signal bram_dout_int       : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0) := (others => '0');
  signal jtag_we_int         : std_logic;
  signal jtag_clk_int        : std_logic;
  signal bram_ce_valid       : std_logic;
  signal din_load            : std_logic;
  --
  signal jtag_dout_0_masked  : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
  signal jtag_dout_1_masked  : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
  signal jtag_dout_2_masked  : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
  signal jtag_dout_3_masked  : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
  signal jtag_dout_4_masked  : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
  signal jtag_dout_5_masked  : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
  signal jtag_dout_6_masked  : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
  signal jtag_dout_7_masked  : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
  signal picoblaze_reset_int : std_logic_vector(C_NUM_PICOBLAZE-1 downto 0) := (others => '0');
  --        
begin
  bus_zero <= (others => '0');
  --
  jtag_loader_gen: if (C_JTAG_LOADER_ENABLE = 1) generate
    --
    -- Insert BSCAN primitive for target device architecture.
    --
    BSCAN_SPARTAN6_gen: if (C_FAMILY="S6") generate
    begin
      BSCAN_BLOCK_inst : BSCAN_SPARTAN6
      generic map ( JTAG_CHAIN => C_JTAG_CHAIN)
      port map( CAPTURE => capture,
                   DRCK => drck,
                  RESET => open,
                RUNTEST => open,
                    SEL => bram_ce_valid,
                  SHIFT => shift,
                    TCK => open,
                    TDI => shift_din,
                    TMS => open,
                 UPDATE => jtag_clk_int,
                    TDO => shift_dout);
    end generate BSCAN_SPARTAN6_gen;   
    --
    BSCAN_VIRTEX6_gen: if (C_FAMILY="V6") generate
    begin
      BSCAN_BLOCK_inst: BSCAN_VIRTEX6
      generic map(    JTAG_CHAIN => C_JTAG_CHAIN,
                    DISABLE_JTAG => FALSE)
      port map( CAPTURE => capture,
                   DRCK => drck,
                  RESET => open,
                RUNTEST => open,
                    SEL => bram_ce_valid,
                  SHIFT => shift,
                    TCK => open,
                    TDI => shift_din,
                    TMS => open,
                 UPDATE => jtag_clk_int,
                    TDO => shift_dout);
    end generate BSCAN_VIRTEX6_gen;   
    --
    BSCAN_7SERIES_gen: if (C_FAMILY="7S") generate
    begin
      BSCAN_BLOCK_inst: BSCANE2
      generic map(    JTAG_CHAIN => C_JTAG_CHAIN,
                    DISABLE_JTAG => "FALSE")
      port map( CAPTURE => capture,
                   DRCK => drck,
                  RESET => open,
                RUNTEST => open,
                    SEL => bram_ce_valid,
                  SHIFT => shift,
                    TCK => open,
                    TDI => shift_din,
                    TMS => open,
                 UPDATE => jtag_clk_int,
                    TDO => shift_dout);
    end generate BSCAN_7SERIES_gen;   
    --
    --
    -- Insert clock buffer to ensure reliable shift operations.
    --
    upload_clock: BUFG
    port map( I => drck,
              O => shift_clk);
    --        
    --        
    --  Shift Register      
    --        
    --
    control_reg_ce_shift: process (shift_clk)
    begin
      if shift_clk'event and shift_clk = '1' then
        if (shift = '1') then
          control_reg_ce <= shift_din;
        end if;
      end if;
    end process control_reg_ce_shift;
    --        
    bram_ce_shift: process (shift_clk)
    begin
      if shift_clk'event and shift_clk='1' then  
        if (shift = '1') then
          if(C_NUM_PICOBLAZE > 1) then
            for i in 0 to C_NUM_PICOBLAZE-2 loop
              bram_ce(i+1) <= bram_ce(i);
            end loop;
          end if;
          bram_ce(0) <= control_reg_ce;
        end if;
      end if;
    end process bram_ce_shift;
    --        
    bram_we_shift: process (shift_clk)
    begin
      if shift_clk'event and shift_clk='1' then  
        if (shift = '1') then
          jtag_we_int <= bram_ce(C_NUM_PICOBLAZE-1);
        end if;
      end if;
    end process bram_we_shift;
    --        
    bram_a_shift: process (shift_clk)
    begin
      if shift_clk'event and shift_clk='1' then  
        if (shift = '1') then
          for i in 0 to C_BRAM_MAX_ADDR_WIDTH-2 loop
            jtag_addr_int(i+1) <= jtag_addr_int(i);
          end loop;
          jtag_addr_int(0) <= jtag_we_int;
        end if;
      end if;
    end process bram_a_shift;
    --        
    bram_d_shift: process (shift_clk)
    begin
      if shift_clk'event and shift_clk='1' then  
        if (din_load = '1') then
          jtag_din_int <= bram_dout_int;
         elsif (shift = '1') then
          for i in 0 to C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-2 loop
            jtag_din_int(i+1) <= jtag_din_int(i);
          end loop;
          jtag_din_int(0) <= jtag_addr_int(C_BRAM_MAX_ADDR_WIDTH-1);
        end if;
      end if;
    end process bram_d_shift;
    --
    shift_dout <= jtag_din_int(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1);
    --
    --
    din_load_select:process (bram_ce, din_load, capture, bus_zero, control_reg_ce) 
    begin
      if ( bram_ce = bus_zero ) then
        din_load <= capture and control_reg_ce;
       else
        din_load <= capture;
      end if;
    end process din_load_select;
    --
    --
    -- Control Registers 
    --
    num_picoblaze <= conv_std_logic_vector(C_NUM_PICOBLAZE-1,3);
    picoblaze_instruction_data_width <= conv_std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1,5);
    --	
    control_registers: process(jtag_clk_int) 
    begin
      if (jtag_clk_int'event and jtag_clk_int = '1') then
        if (bram_ce_valid = '1') and (jtag_we_int = '0') and (control_reg_ce = '1') then
          case (jtag_addr_int(3 downto 0)) is 
            when "0000" => -- 0 = version - returns (7 downto 4) illustrating number of PB
                           --               and (3 downto 0) picoblaze instruction data width
                           control_dout_int <= num_picoblaze & picoblaze_instruction_data_width;
            when "0001" => -- 1 = PicoBlaze 0 reset / status
                           if (C_NUM_PICOBLAZE >= 1) then 
                            control_dout_int <= picoblaze_reset_int(0) & "00" & (conv_std_logic_vector(C_ADDR_WIDTH_0-1,5) );
                           else 
                            control_dout_int <= (others => '0');
                           end if;
            when "0010" => -- 2 = PicoBlaze 1 reset / status
                           if (C_NUM_PICOBLAZE >= 2) then 
                             control_dout_int <= picoblaze_reset_int(1) & "00" & (conv_std_logic_vector(C_ADDR_WIDTH_1-1,5) );
                            else 
                             control_dout_int <= (others => '0');
                           end if;
            when "0011" => -- 3 = PicoBlaze 2 reset / status
                           if (C_NUM_PICOBLAZE >= 3) then 
                            control_dout_int <= picoblaze_reset_int(2) & "00" & (conv_std_logic_vector(C_ADDR_WIDTH_2-1,5) );
                           else 
                            control_dout_int <= (others => '0');
                           end if;
            when "0100" => -- 4 = PicoBlaze 3 reset / status
                           if (C_NUM_PICOBLAZE >= 4) then 
                            control_dout_int <= picoblaze_reset_int(3) & "00" & (conv_std_logic_vector(C_ADDR_WIDTH_3-1,5) );
                           else 
                            control_dout_int <= (others => '0');
                           end if;
            when "0101" => -- 5 = PicoBlaze 4 reset / status
                           if (C_NUM_PICOBLAZE >= 5) then 
                            control_dout_int <= picoblaze_reset_int(4) & "00" & (conv_std_logic_vector(C_ADDR_WIDTH_4-1,5) );
                           else 
                            control_dout_int <= (others => '0');
                           end if;
            when "0110" => -- 6 = PicoBlaze 5 reset / status
                           if (C_NUM_PICOBLAZE >= 6) then 
                            control_dout_int <= picoblaze_reset_int(5) & "00" & (conv_std_logic_vector(C_ADDR_WIDTH_5-1,5) );
                           else 
                            control_dout_int <= (others => '0');
                           end if;
            when "0111" => -- 7 = PicoBlaze 6 reset / status
                           if (C_NUM_PICOBLAZE >= 7) then 
                            control_dout_int <= picoblaze_reset_int(6) & "00" & (conv_std_logic_vector(C_ADDR_WIDTH_6-1,5) );
                           else 
                            control_dout_int <= (others => '0');
                           end if;
            when "1000" => -- 8 = PicoBlaze 7 reset / status
                           if (C_NUM_PICOBLAZE >= 8) then 
                            control_dout_int <= picoblaze_reset_int(7) & "00" & (conv_std_logic_vector(C_ADDR_WIDTH_7-1,5) );
                           else 
                            control_dout_int <= (others => '0');
                           end if;
            when "1111" => control_dout_int <= conv_std_logic_vector(C_BRAM_MAX_ADDR_WIDTH -1,8);
            when others => control_dout_int <= (others => '1');
          end case;
        else 
          control_dout_int <= (others => '0');
        end if;
      end if;
    end process control_registers;
    -- 
    control_dout(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-8) <= control_dout_int;
    --
    pb_reset: process(jtag_clk_int) 
    begin
      if (jtag_clk_int'event and jtag_clk_int = '1') then
        if (bram_ce_valid = '1') and (jtag_we_int = '1') and (control_reg_ce = '1') then
          picoblaze_reset_int(C_NUM_PICOBLAZE-1 downto 0) <= control_din(C_NUM_PICOBLAZE-1 downto 0);
        end if;
      end if;
    end process pb_reset;    
    --
    --
    -- Assignments 
    --
    control_dout (C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-9 downto 0) <= (others => '0') when (C_PICOBLAZE_INSTRUCTION_DATA_WIDTH > 8);
    --
    -- Qualify the blockram CS signal with bscan select output
    jtag_en_int <= bram_ce when bram_ce_valid = '1' else (others => '0');
    --      
    jtag_en_expanded(C_NUM_PICOBLAZE-1 downto 0) <= jtag_en_int;
    jtag_en_expanded(7 downto C_NUM_PICOBLAZE) <= (others => '0') when (C_NUM_PICOBLAZE < 8);
    --        
    bram_dout_int <= control_dout or jtag_dout_0_masked or jtag_dout_1_masked or jtag_dout_2_masked or jtag_dout_3_masked or jtag_dout_4_masked or jtag_dout_5_masked or jtag_dout_6_masked or jtag_dout_7_masked;
    --
    control_din <= jtag_din_int;
    --        
    jtag_dout_0_masked <= jtag_dout_0 when jtag_en_expanded(0) = '1' else (others => '0');
    jtag_dout_1_masked <= jtag_dout_1 when jtag_en_expanded(1) = '1' else (others => '0');
    jtag_dout_2_masked <= jtag_dout_2 when jtag_en_expanded(2) = '1' else (others => '0');
    jtag_dout_3_masked <= jtag_dout_3 when jtag_en_expanded(3) = '1' else (others => '0');
    jtag_dout_4_masked <= jtag_dout_4 when jtag_en_expanded(4) = '1' else (others => '0');
    jtag_dout_5_masked <= jtag_dout_5 when jtag_en_expanded(5) = '1' else (others => '0');
    jtag_dout_6_masked <= jtag_dout_6 when jtag_en_expanded(6) = '1' else (others => '0');
    jtag_dout_7_masked <= jtag_dout_7 when jtag_en_expanded(7) = '1' else (others => '0');
    --
    jtag_en <= jtag_en_int;
    jtag_din <= jtag_din_int;
    jtag_addr <= jtag_addr_int;
    jtag_clk <= jtag_clk_int;
    jtag_we <= jtag_we_int;
    picoblaze_reset <= picoblaze_reset_int;
    --        
  end generate jtag_loader_gen;
--
end Behavioral;
--
--
------------------------------------------------------------------------------------
--
-- END OF FILE m25p16_spi_uart_bridge.vhd
--
------------------------------------------------------------------------------------
